package com.accenture.techlabs.assist.profile;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;

public class MockDataManager implements IDataManager {

	private int sequence = 0;
	private final Object readlock = new Object();

	private BufferedReader reader;
	private String dirname = ".";
	

	public String getRepositoryDirname() {
		return dirname;
	}

	public void setRepositoryDirname(String dirname) {
		this.dirname = dirname;
	}

	private boolean openFile() {
		File repository_dir = new File(dirname);
		File[] files = null;
		for (int retry = 10; retry>0; retry--) {
			FilenameFilter dew = new FilenameFilter() {
				public boolean accept(File funtoosh, String name) {
					return name.startsWith("P" + sequence + "_")
							&& name.endsWith(".txt");
				}
			};

			files = repository_dir.listFiles(dew);
			if (files == null){
				System.err.println("Error Reading Directory: "+dirname);
				return false;
			}
			sequence++;
			if (files.length>0 && files[0]!=null)
				break;
		};

		if (files.length==0)
			return false;

		try {
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(files[0])));
		} catch (FileNotFoundException e) {
			System.err.println("Can not locate profile data file "
					+ files[0]);
			return false;
		}
		return true;
	}

	@Override
	public String getNextLine() throws InterruptedException {
		synchronized (readlock) {
			for (;;) {
				if (reader == null) {
					boolean newfileopened = openFile();
					if (!newfileopened)
						return null;
				}
				String val = null;
				try {
					val = reader.readLine();
				} catch (IOException e) {
					System.err.println("Error Reading Profile infomation!");
					e.printStackTrace();
					val = null;
				}

				if (val != null)
					return val;
				else {
					reader = null;
				}
			}
		}
	}

	@Override
	public void insertLine(String data) {
		throw new RuntimeException("Wrong Module! This is a static repository that reads from disk, which would not accept any input.");
	}

	@Override
	public void shutDown() {

	}

}
