package com.accenture.techlabs.assist.profile;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.jmeter.JMeter;
import org.apache.log4j.Logger;

public class QueueDataManager implements IDataManager {

	// QI
	private static final Logger logger = Logger.getLogger(JMeter.class);
	
	private final Object readlock = new Object();
	private final Object writelock = new Object();
	private boolean shuttingdown = false;

	/**
	 * field for writing to a profile archive file
	 */
	private PrintWriter archive = null;
	public String archivename = null;
	/**
	 * field for reading from a profile archive file
	 */
	private BufferedReader reader = null;

	/**
	 * This field store a list of file names of profiles waiting to be processed
	 */
	private LinkedBlockingQueue<String> profilenames = new LinkedBlockingQueue<String>();

	QueueDataManager() {
		//detectExistingProfiles();
		newProfile();
		//ProfRepository.startProfile("2");
	}


	private final int FILE_RETRY = 5;
	private void detectExistingProfiles(){
		if (archive!=null || sequence >0 || profilenames.size()>0)
			return; // This only runs before everything else started;
		File repository_dir = new File(ProfRepository.profiledir);
		int failedAttempt = 0;
		for (int seq=0; ; seq++){
			if (failedAttempt >= FILE_RETRY)
				break;

			File[] files = null;
			final String prefix = "P" + seq + "_";
			FilenameFilter dew = new FilenameFilter() {
				public boolean accept(File funtoosh, String name) {
					return name.startsWith(prefix) && name.endsWith(".txt");
				}
			};

			files = repository_dir.listFiles(dew);
			if (files == null || files.length==0 || files[0]==null) {
				failedAttempt++;
				continue;
			}
			profilenames.add(files[0].getAbsolutePath());
			sequence = seq+1;
			//TODO!! is there also a way to remove the last file from queue and also delete it? since chances are this profile has been corrupted?
		}
	}

	private int sequence = 0;
	public  void newProfile() {
		if (archive != null) {
			archive.close();
			profilenames.add(archivename);
		}

		try {
			File tempfile = File
					.createTempFile("P"+sequence+"_", ".txt", new File(ProfRepository.profiledir));
			//logger.info("create a new profile in the repository!"); // QI
			archive = new PrintWriter(new FileOutputStream(tempfile));
			archivename = tempfile.getAbsolutePath();
			sequence++;
		} catch (IOException e) {
			throw new RuntimeException("Disk Full! Fail to write to profile repository!");
		}
	}

	public String getNextLine() throws InterruptedException {
		synchronized (readlock) {
			String val = null;
			for (;;) {
				if (reader == null) {
					String filename = profilenames.take(); // maybe blocked if 
					// profile is not available yet
					try {
						reader = new BufferedReader(new InputStreamReader(
								new FileInputStream(filename)));
					} catch (FileNotFoundException e) {
						System.err.println("Corrupted Profile Repository!");
						e.printStackTrace();
						continue; // retry to get next profile archive
					}
				}

				try {
					val = reader.readLine();
				} catch (IOException e) {
					System.err.println("Error Reading Profile infomation!");
					e.printStackTrace();
					val = null;
				}

				if (val != null)
					return val;
				else{
					// now val is null, meaning there's no more data in the current data file.
					if (shuttingdown)
						return null;
					reader = null; // so that it will get into the branch in the
				// beginning of the loop and get the next
				// profile archive to read
				}
			}
		}
	}

	public void insertLine(String data) {
		if (shuttingdown){
			System.err.println("Error: Trying to write to repository after shutting down!!!");
			return;
		}
		synchronized (writelock) {
			if(data.contains("START_")){
				System.out.println(data);
			}
			archive.println(data); // write into the profile by PrintWriter; QI
			if (data.startsWith(ProfRepository.profileEndTag)) {
				//logger.info("insertLine END_PROFILE! new a profile! "); //QI
				newProfile();
			}
		}
	}

	public void shutDown() {
		shuttingdown = true;
		if (archive != null) {
			archive.close();
			profilenames.add(archivename);
		}		
	}

}
