package forepost;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;
import java.util.logging.Level;


public class DataArffManager extends ArffManager
{
	protected Hashtable<String, ArrayList<String>> UniqueKeysValues = new Hashtable<String, ArrayList<String>>();
//	protected Hashtable<String, String> UniqueKVPairs = new Hashtable<String, String>(); 
	private ArrayList<String> Values = null;
	private enum TYPEOFVALUES{NUMERICVAL, REALVAL, STRINGVAL}
	private Hashtable <String, TYPEOFVALUES> typeOfValues = new Hashtable <String, TYPEOFVALUES>();  
	private HashSet<String> multipleValueKeys = new HashSet<String>();
	private ScreenInputValues TheMap = new ScreenInputValues();

	public DataArffManager( String arffFileName, LinkedList <ProfileParser> table ) throws Exception
	{
		super(arffFileName, table);
		multipleValueKeys.clear();
	}
	
	public void AddScreenInputValueMap( String screen, String input, String value ) throws Exception
	{
		String key = input;
		if( key != null && key.toLowerCase().endsWith("unselected") ){
			key = key.toLowerCase();
			int ind = key.indexOf("unselected");
			key = key.substring(0, ind);
		}
		
		TheMap.AddNewValue(screen, key, value);
	}
	
	public void AddUniqueKeyValue( String key, String val ) throws Exception
	{
		if( key == null ){
			throw new Exception("Key string cannot be null");
		}

		if( val == null ){
			throw new Exception("Value string cannot be null");
		}
		
		if( val.length() == 0 || key.length() == 0 ){
			return;
		}
		
		if( key.toLowerCase().endsWith("unselected") ){
			key = key.toLowerCase();
			int ind = key.indexOf("unselected");
			key = key.substring(0, ind);
		}

		
		ArrayList<String> Vals = UniqueKeysValues.get(key);
		if( Vals == null ){
			Vals = new ArrayList<String>();
			Vals.add(val);
			UniqueKeysValues.put(key, Vals);
//			System.out.println("Adding the first value for the key " + key );
		} else {
			Iterator<String> it = Vals.iterator();
			while( it.hasNext() ){
				String kv = it.next();
				if( kv.compareTo(val) == 0 ){
					return;
				}
			}
//			System.out.println("There are " + Vals.size() + " values for the key " + key );
			Vals.add(val);
			
		}
	}
	
	private boolean checkIfAttributeIsNumeric(ArrayList<String> attrVals) {
		boolean numeric = false;
		try {
			if(attrVals.size() > 0 ){
				Iterator<String> it = attrVals.iterator();
				while(it.hasNext()){
					Integer.parseInt(it.next());
				}
				numeric = true;
			}
		} catch (NumberFormatException ex) {
		}
       
		return numeric;
	}

	private boolean checkIfAttributeIsBoolean(ArrayList<String> attrVals) {
		boolean boolval = true;
		if(attrVals.size() > 0 ){
			Iterator<String> it = attrVals.iterator();
			if(!it.hasNext())
				boolval = false;
			while(it.hasNext()){
				String s = it.next();
				if( s != null && s.compareToIgnoreCase("y") != 0 && s.compareToIgnoreCase("n") != 0 )
				{
					boolval = false;
					break;
				}
			}
		}
		else
			boolval = false;
       
		return boolval;
	}
	
	private boolean checkIfAttributeIsReal(ArrayList<String> attrVals) {
		boolean numeric = false;
		try {
			if(attrVals.size() > 0 ){
				Iterator<String> it = attrVals.iterator();
				while(it.hasNext()){
					Float.parseFloat(it.next());
				}
				numeric = true;
			}
		} catch (NumberFormatException ex) {
		}
       
		return numeric;
	}
	

	public void CreateArffData(String relationName, ArffManager.KINDOFDATA kod, int numProfilesFromTable ) throws Exception 
	{
		InputsExclusion exclude = new InputsExclusion();
		super.CreateArffData(relationName);
		multipleValueKeys.clear();
		ArrayList<Hashtable<String, String>> mergedTables = new ArrayList<Hashtable<String, String>>(); 
		
		try {
			String key = null, outStr = null;
//			for(int i = 0; i < profiles.size(); i++ ){
			for(int i = 0; i < numProfilesFromTable; i++ ){
				ProfileParser pp = profiles.get(i);
				if( pp!= null )
				{
					ArrayList<Hashtable<String, String>> dataRuns = null;
					if( kod == ArffManager.KINDOFDATA.IMS ){
						dataRuns = pp.getImsRuns();
					} else if( kod == ArffManager.KINDOFDATA.INPUTPARAMS ){
						dataRuns = pp.getInputParamsRuns();
					}
					
					Hashtable<String, String> mergedTable = MergeRuns4Profile(dataRuns);
					mergedTables.add(mergedTable);
				}
			}
			
			if( mergedTables.size() <= 0 ){
				FPUtility.getLogger().log(Level.FINE, "No data for the ARFF file");
				return;
			}

			
			Enumeration<String> keys = UniqueKeysValues.keys();
			while(keys.hasMoreElements()){
				key = keys.nextElement();
				String [] checkIt = key.split("\\.");
				if( checkIt.length != 2 ){
					continue;
				}
				
				if( !exclude.IsExcluded(checkIt[0].toLowerCase(), checkIt[1].toLowerCase()) )
				{
					Values = UniqueKeysValues.get(key);
					if(Values.size() > 0 ){
						if( checkIfAttributeIsNumeric(Values) ){
							outStr = ATTRIBUTE + key + " NUMERIC\n";
							typeOfValues.put(key, TYPEOFVALUES.NUMERICVAL);
						}
						else if( checkIfAttributeIsReal(Values) ){
							outStr = ATTRIBUTE + key + " REAL\n";
							typeOfValues.put(key, TYPEOFVALUES.REALVAL);
						}
						else {
							outStr = ATTRIBUTE + key + " { ";
							typeOfValues.put(key, TYPEOFVALUES.STRINGVAL);
							Iterator<String> it = Values.iterator();
							boolean first = true;
							while(it.hasNext()){
								if(!first){
									outStr += ", ";
								}else {
									first = false;
								}
								outStr += "\"";
								outStr += it.next();
								outStr += "\"";
							}
							outStr += " }\n";
						}
						output.write( outStr );
					}
				}
			}
			
			outStr = ATTRIBUTE + " class { ";
			Iterator<String> iter = Classes.iterator();
			while( iter.hasNext() ){
				outStr += iter.next();
				if( iter.hasNext() )
					outStr += ", ";
			}
			outStr += " }\n\n";
			output.write( outStr );
			
			PopulateData(kod, mergedTables);

			output.flush();
			output.close();
//			System.out.println(relationName + " arff is produced"); hide_focus
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void CreateScreenInputDataXml()
	{
		InputsExclusion exclude = new InputsExclusion();
		try {
			String xmlFileName = FPUtility.getScreenInputValuesFile();
			String output = "<inputs>\n";
			HashMap<String, HashMap<String, ArrayList<String>>> sivMap = TheMap.getTheMap();
			Set<String> screens = sivMap.keySet();
			Iterator<String> itScreenKeys = screens.iterator();
			while(itScreenKeys.hasNext()){
				String screen = itScreenKeys.next();
				HashMap<String, ArrayList<String>> inputValuesMap = sivMap.get(screen);
				output += "<screen name=\"";
				output += screen;
				output += "\">\n";
				Set<String> inputs = inputValuesMap.keySet();
				Iterator<String> itInputKeys = inputs.iterator();
				while(itInputKeys.hasNext()){
					String input = itInputKeys.next();
					if( !exclude.IsExcluded(screen, input) )
					{
						output += "<input name=\"";
						output += input.toLowerCase();
						output += "\">\n";
						ArrayList<String> values = inputValuesMap.get(input);
						Iterator<String> itValues = values.iterator();
						while(itValues.hasNext()){
							String value = itValues.next();
							output += "<value>";
							output += value;
							output += "</value>\n";
						}
						output += "</input>\n";
					}
				}
				output += "</screen>\n";
			}
			output += "</inputs>\n";
			BufferedWriter xmlOut = new BufferedWriter(new FileWriter(xmlFileName));
			xmlOut.write(output);
			xmlOut.flush();
			xmlOut.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}		
	
	public void CreateACTSInputData()
	{
		InputsExclusion exclude = new InputsExclusion();
		try {
			String acstFileName = FPUtility.getScreenInputACSTValuesFile();
			String output = "[System]\nName: Test configuration for Renters\n";
			output += "[Parameter]\n"; 
			HashMap<String, HashMap<String, ArrayList<String>>> sivMap = TheMap.getTheMap();
			Set<String> screens = sivMap.keySet();
			Iterator<String> itScreenKeys = screens.iterator();
			while(itScreenKeys.hasNext()){
				String screen = itScreenKeys.next();
				HashMap<String, ArrayList<String>> inputValuesMap = sivMap.get(screen);
				Set<String> inputs = inputValuesMap.keySet();
				Iterator<String> itInputKeys = inputs.iterator();
				while(itInputKeys.hasNext()){
					String input = itInputKeys.next();
					if( !exclude.IsExcluded(screen, input) )
					{
						output += screen.toLowerCase() + "." + input.toLowerCase();
						ArrayList<String> values = inputValuesMap.get(input);
						if( checkIfAttributeIsNumeric(values) ){
							output +=  " (int):";
						}
						else {
							output +=  " (enum):";
						}
						
						Iterator<String> itValues = values.iterator();
						while(itValues.hasNext()){
							String value = itValues.next();
							output += value;
							if( itValues.hasNext() )
								output += ",";
						}
						output += "\n";
					}
				}
			}
			BufferedWriter xmlOut = new BufferedWriter(new FileWriter(acstFileName));
			xmlOut.write(output);
			xmlOut.flush();
			xmlOut.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void CreateCSVInputData()
	{
		InputsExclusion exclude = new InputsExclusion();
		try {
			String acstFileName = FPUtility.getScreenInputCSVValuesFile();
			String output = "";//"[System]\nName: Test configuration for Renters\n";
//			output += "[Parameter]\n"; 
			HashMap<String, HashMap<String, ArrayList<String>>> sivMap = TheMap.getTheMap();
			Set<String> screens = sivMap.keySet();
			Iterator<String> itScreenKeys = screens.iterator();
			while(itScreenKeys.hasNext()){
				String screen = itScreenKeys.next();
				HashMap<String, ArrayList<String>> inputValuesMap = sivMap.get(screen);
				Set<String> inputs = inputValuesMap.keySet();
				Iterator<String> itInputKeys = inputs.iterator();
				while(itInputKeys.hasNext()){
					String input = itInputKeys.next();
					if( !exclude.IsExcluded(screen, input) ){
						output += screen.toLowerCase() + "." + input.toLowerCase();
						ArrayList<String> values = inputValuesMap.get(input);
						if( checkIfAttributeIsNumeric(values) ){
							output +=  ",int";
						}
						else if( checkIfAttributeIsBoolean(values) ){
							output +=  ",boolean";
						}
						else {
							output +=  ",enum";
						}
						
						Iterator<String> itValues = values.iterator();
						while(itValues.hasNext()){
							String value = itValues.next();
							output += "," + value;
						}
						output += "\n";
					}
				}
			}
			BufferedWriter xmlOut = new BufferedWriter(new FileWriter(acstFileName));
			xmlOut.write(output);
			xmlOut.flush();
			xmlOut.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void PopulateDataByRunsPerProfile(ArffManager.KINDOFDATA kod)
	{
		boolean firstentry = true;
		try {
			String outStr = "@data\n";
			output.write( outStr );
			
			for(int i = 0; i < profiles.size(); i++ ){
				ProfileParser pp = profiles.get(i);
				if( pp!= null )
				{
					ArrayList<Hashtable<String, String>> dataRuns = null;
					if( kod == ArffManager.KINDOFDATA.IMS ){
						dataRuns = pp.getImsRuns();
					} else if( kod == ArffManager.KINDOFDATA.INPUTPARAMS ){
						dataRuns = pp.getInputParamsRuns();
					}
					Iterator<Hashtable<String, String>> itr = dataRuns.iterator();
					while(itr.hasNext()){
						Hashtable<String, String> tbl = itr.next();
						Enumeration<String> keys = UniqueKeysValues.keys();
						firstentry = true;
						while(keys.hasMoreElements()){
							String key = keys.nextElement();
							String val = tbl.get(key);
							if( val == null )
								val = "?";
							if( val.length() == 0 ){
								val = "?";
							}
							if( !firstentry ){
								if( val.compareTo("?") != 0 ){
									if( typeOfValues.get(key) == TYPEOFVALUES.STRINGVAL ){
										outStr += ", " + "\"" + val + "\"";
									}
									else {
										outStr += ", " + val;
									}
								} else {
									outStr += ", " + val;
								}
							}
							else{
								firstentry = false;
								if( val.compareTo("?") != 0 ){
									if( typeOfValues.get(key) == TYPEOFVALUES.STRINGVAL ){
										outStr = "\"" + val + "\"";
									}
									else {
										outStr = val;
									}
								} else {
									outStr = val;
								}
							}
						}
						//class entry finishes the data line
						outStr += ", " + pp.getProfileClass();
						outStr += "\n";
						output.write( outStr );
					}
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	private void PopulateData(ArffManager.KINDOFDATA kod, ArrayList<Hashtable<String, String>> mergedTables) throws Exception
	{
		InputsExclusion exclude = new InputsExclusion();
		boolean firstentry = true;
		try {
			String outStr = "@data\n";
			output.write( outStr );
/*			
			for(int i = 0; i < profiles.size(); i++ ){
				ProfileParser pp = profiles.get(i);
				if( pp!= null )
				{
					ArrayList<Hashtable<String, String>> dataRuns = null;
					if( kod == ArffManager.KINDOFDATA.IMS ){
						dataRuns = pp.getImsRuns();
					} else if( kod == ArffManager.KINDOFDATA.INPUTPARAMS ){
						dataRuns = pp.getInputParamsRuns();
					}
					
					Hashtable<String, String> mergedTable = MergeRuns4Profile(dataRuns);
					mergedTables.add(mergedTable);
				}
			}
*/			
			if( mergedTables.size() <= 0 ){
				FPUtility.getLogger().log(Level.FINE, "No data for the ARFF file");
				return;
			}

			int i = -1;
			Iterator<Hashtable<String, String>> itr = mergedTables.iterator();
			while(itr.hasNext()){
				++i;
				if( i >= profiles.size() )
					break;
				ProfileParser pp = profiles.get(i);
				if( pp.getProfileClass() == null || pp.getProfileClass().compareTo(FPUtility.UNASSIGNEDCLASS) == 0 )
					continue;
				Hashtable<String, String> tbl = itr.next();
				if( tbl.size() <= 0 ) continue;
				Enumeration<String> keys = UniqueKeysValues.keys();
				firstentry = true;
				while(keys.hasMoreElements()){
					String key = keys.nextElement();
					String [] checkIt = key.split("\\.");
					if( checkIt.length != 2 ){
						continue;
					}
					
					if( !exclude.IsExcluded(checkIt[0].toLowerCase(), checkIt[1].toLowerCase()) )
					{
						String val = tbl.get(key);
						if( val == null )
							val = "?";
						if( val.length() == 0 ){
							val = "?";
						}
						if( !firstentry ){
							if( val.compareTo("?") != 0 ){
								if( typeOfValues.get(key) == TYPEOFVALUES.STRINGVAL ){
									outStr += ", " + "\"" + val + "\"";
								}
								else {
									outStr += ", " + val;
								}
							} else {
								outStr += ", " + val;
							}
						}
						else{
							firstentry = false;
							if( val.compareTo("?") != 0 ){
								if( typeOfValues.get(key) == TYPEOFVALUES.STRINGVAL ){
									outStr = "\"" + val + "\"";
								}
								else {
									outStr = val;
								}
							} else {
								outStr = val;
							}
						}
					}
				}
				//class entry finishes the data line
				outStr += ", " + pp.getProfileClass();
				outStr += "\n";
				output.write( outStr );
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	Hashtable<String, String> MergeRuns4Profile(ArrayList<Hashtable<String, String>> dataRuns) throws Exception
	{
		HashSet<String> multipleValueProfileKeys = new HashSet<String>();
		Hashtable<String, String> MergedTable = new Hashtable<String, String>();
		String key = null;
		
		if( dataRuns == null ){
			FPUtility.getLogger().log(Level.FINE, "Datarun object is null from the profile parser");
			return( MergedTable );
		}
		
		multipleValueProfileKeys.clear();
		
		Iterator<Hashtable<String, String>> itr = dataRuns.iterator();
		while(itr.hasNext()){
			Hashtable<String, String> tbl = itr.next();
			Set<String> set = tbl.keySet();

		    Iterator<String> itrTbl = set.iterator();
		    while( itrTbl.hasNext() ) {
		      key = itrTbl.next();
		      if(multipleValueKeys.contains(key)){
		    	  continue;
		      }
		      else {
			      if( !multipleValueProfileKeys.contains(key) ) {
			    	  multipleValueProfileKeys.add(key);
			    	  MergedTable.put(key, tbl.get(key));
			      }
			      else {
//			    	  System.out.println("Multiple values for the key " + key ); hide_focus
			    	  multipleValueKeys.add(key);
			    	  MergedTable.remove(key);
			    	  UniqueKeysValues.remove(key);
			      }
		      }
		    }			
			
//			multipleValueKeys.add(key);
		}
		
		return( MergedTable );
	}
}
