/*
 * FOREPOSTView.java
 */

package forepost;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.logging.Level;

import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.Timer;
import javax.swing.table.DefaultTableModel;

import org.apache.log4j.Logger;
import org.jdesktop.application.Action;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.TaskMonitor;

import com.accenture.techlabs.assist.profile.ProfRepository;
/**
 * The application's main frame.
 */
public class FOREPOSTView extends FrameView {
	// QI
	private static final Logger logger = Logger.getLogger(ProfRepository.class);
	
    private ProfileAnalyzer analyzer = null;
    private AnalyzerICA analyzerICA = null;
    private boolean autocluster =  false;
    private boolean bNewClassIsBeingAdded = false;
    javax.swing.table.DefaultTableModel profileTableModel = new javax.swing.table.DefaultTableModel();
    javax.swing.table.DefaultTableModel ruleTableModel = new javax.swing.table.DefaultTableModel();
    javax.swing.table.DefaultTableModel bottleneckTableModel = new javax.swing.table.DefaultTableModel();
    
    public javax.swing.table.DefaultTableModel getProfileTableModel() {
		return profileTableModel;
	}

    public javax.swing.table.DefaultTableModel getRuleTableModel() {
		return ruleTableModel;
	}
    
    public javax.swing.table.DefaultTableModel getBottleNeckModel() {
    	return bottleneckTableModel;
    }
    

	public FOREPOSTView(SingleFrameApplication app, ProfileAnalyzer pa) {
        super(app);
        setAnalyzer(pa);
        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    

	@Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = FOREPOSTApp.getApplication().getMainFrame();
            aboutBox = new FOREPOSTAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        FOREPOSTApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        statusTabPanel = new javax.swing.JPanel();
        statusLabel = new javax.swing.JLabel();
        runsLabel = new javax.swing.JLabel();
        methodLabel = new javax.swing.JLabel();
        attributeLabel = new javax.swing.JLabel();
        unusedInputLabel = new javax.swing.JLabel();
        rulesLearntLabel = new javax.swing.JLabel();
        rulesUnlearntLabel = new javax.swing.JLabel();
        statusValue = new javax.swing.JLabel();
        runsValue = new javax.swing.JLabel();
        executedLabel = new javax.swing.JLabel();
        timeLabel = new javax.swing.JLabel();
        fromLabel = new javax.swing.JLabel();
        classLabel = new javax.swing.JLabel();
        methodValue = new javax.swing.JLabel();
        executionValue = new javax.swing.JLabel();
        classValue = new javax.swing.JLabel();
        attributesValue = new javax.swing.JLabel();
        usedLabel = new javax.swing.JLabel();
        usedValue = new javax.swing.JLabel();
        dataTransfered = new javax.swing.JLabel();
        usedInputValue = new javax.swing.JLabel();
        rulesLearntValue = new javax.swing.JLabel();
        rulesUnlearntValue = new javax.swing.JLabel();
        inputsUsedLabel = new javax.swing.JLabel();
        rulesLearntInputUsedValue = new javax.swing.JLabel();
        rulesUnlearntInputsLabel = new javax.swing.JLabel();
        rulesUnlearntInputsValue = new javax.swing.JLabel();
        rulesTabPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        JCheckBox checkBox2 = new javax.swing.JCheckBox();
        rulesTable = new javax.swing.JTable();
        profileTabPanel = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        JCheckBox excludeCheckBox = new javax.swing.JCheckBox();
        //JComboBox 
        combo = new ClassComboBox(new String[]{FPUtility.UNASSIGNEDCLASS},-1);
        profileTable = new javax.swing.JTable();
        configTabPanel = new javax.swing.JPanel();
        jLabel16 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        categoriesInputList = new javax.swing.JList();
        addConfigInputs = new javax.swing.JButton();
        removeConfigInputs = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JSeparator();
        packagesLabel = new javax.swing.JLabel();
        configurationClassLabel = new javax.swing.JLabel();
        configurationMethodLabel = new javax.swing.JLabel();
        configurationAttributesLabel = new javax.swing.JLabel();
        configExcludeButton = new javax.swing.JButton();
        configIncludeButton = new javax.swing.JButton();
        recommendationTextArea = new javax.swing.JTextField();
        configMethodTextArea = new javax.swing.JTextField();
        configMethodTextArea = new javax.swing.JTextField();
        ConfigAttrTextArea = new javax.swing.JTextField();
        jSeparator2 = new javax.swing.JSeparator();
        classificationClassLabel = new javax.swing.JLabel();
        jScrollPane4 = new javax.swing.JScrollPane();
        configClassificationClassList = new javax.swing.JList();
        deleteButton = new javax.swing.JButton();
        addButton = new javax.swing.JButton();
        newCategoryLabel = new javax.swing.JLabel();
        newClassTextArea = new javax.swing.JTextField();
        analysisTabPanel = new javax.swing.JPanel();
        bottleneckTabPanel = new javax.swing.JPanel();
        jScrollPane5 = new javax.swing.JScrollPane();
        bottleneckTable = new javax.swing.JTable();
        recommendationTabPanel = new javax.swing.JPanel();
        jScrollPane6 = new javax.swing.JScrollPane();
        configPackagesText = new javax.swing.JTextArea();
        menuBar = new javax.swing.JMenuBar();
        autoClusterButton = new javax.swing.JButton();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        fileLoadMenu = new javax.swing.JMenu();
        loadProfileStatsMenu = new javax.swing.JMenuItem();
        loadDataAttributesMenu = new javax.swing.JMenuItem();
        loadInputDataMenu = new javax.swing.JMenuItem();
        controlMenu = new javax.swing.JMenu();
        analyzerMenu = new javax.swing.JMenu();
        analyzerStartMenu = new javax.swing.JMenuItem();
        analyzerPauseMenu = new javax.swing.JMenuItem();
        analyzerResumeMenu = new javax.swing.JMenuItem();
        persistMenu = new javax.swing.JMenu();
        persistARFFMenu = new javax.swing.JMenuItem();
        persistCallgraphMenu = new javax.swing.JMenuItem();
        persistLearnMenu = new javax.swing.JMenuItem();
        persistResultMenu = new javax.swing.JMenuItem();
        wekaMenu = new javax.swing.JMenu();
        wekaStartMenu = new javax.swing.JMenuItem();
        wekaPauseMenu = new javax.swing.JMenuItem();
        wekaResumeMenu = new javax.swing.JMenuItem();
        actionMenu = new javax.swing.JMenu();
        rulesMenu = new javax.swing.JMenu();
        rulesRelearnMenu = new javax.swing.JMenuItem();
        rulesUpdateMenu = new javax.swing.JMenuItem();
        threadMenu = new javax.swing.JMenu();
        threadStartMenu = new javax.swing.JMenuItem();
        threadStopMenu = new javax.swing.JMenuItem();
        threadSuspendMenu = new javax.swing.JMenuItem();
        profileMenu = new javax.swing.JMenu();
        profileAutoClusterMenu = new javax.swing.JMenuItem();
        profileManualClusterMenu = new javax.swing.JMenuItem();
        bottleneckMenu = new javax.swing.JMenu();
        bottleneckExcludeMenu = new javax.swing.JMenuItem();
        bottleneckViewMenu = new javax.swing.JMenuItem();
        bottleneckCallGraphMenu = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();

        mainPanel.setName("mainPanel"); // NOI18N

        jTabbedPane1.setName("jTabbedPane1"); // NOI18N

        statusTabPanel.setName("rulesPanel"); // NOI18N
        statusTabPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(forepost.FOREPOSTApp.class).getContext().getResourceMap(FOREPOSTView.class);
        statusLabel.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        statusLabel.setName("statusLabel"); // NOI18N
        statusTabPanel.add(statusLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 11, 72, 19));

        runsLabel.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        runsLabel.setName("runsLabel"); // NOI18N
        statusTabPanel.add(runsLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 36, -1, -1));

        methodLabel.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        methodLabel.setName("methodLabel"); // NOI18N
        statusTabPanel.add(methodLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 69, -1, -1));

        attributeLabel.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        attributeLabel.setName("attributeLabel"); // NOI18N
        statusTabPanel.add(attributeLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 136, -1, -1));

        unusedInputLabel.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        unusedInputLabel.setName("unusedInputLabel"); // NOI18N
        statusTabPanel.add(unusedInputLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 168, -1, -1));

        rulesLearntLabel.setText(resourceMap.getString("jLabel6.text")); // NOI18N
        rulesLearntLabel.setName("rulesLearntLabel"); // NOI18N
        statusTabPanel.add(rulesLearntLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 200, -1, -1));

        rulesUnlearntLabel.setText(resourceMap.getString("jLabel7.text")); // NOI18N
        rulesUnlearntLabel.setName("rulesUnlearntLabel"); // NOI18N
        statusTabPanel.add(rulesUnlearntLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 250, -1, -1));

        statusValue.setText(resourceMap.getString("jLabel8.text")); // NOI18N
        statusValue.setName("statusValue"); // NOI18N
        statusTabPanel.add(statusValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 11, 156, -1));

        runsValue.setText(resourceMap.getString("jLabel9.text")); // NOI18N
        runsValue.setName("runsValue"); // NOI18N
        statusTabPanel.add(runsValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 43, 142, -1));

        executedLabel.setText(resourceMap.getString("jLabel10.text")); // NOI18N
        executedLabel.setName("executedLabel"); // NOI18N
        statusTabPanel.add(executedLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(334, 69, -1, -1));

        timeLabel.setText(resourceMap.getString("jLabel11.text")); // NOI18N
        timeLabel.setName("timeLabel"); // NOI18N
        statusTabPanel.add(timeLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(574, 75, -1, -1));

        fromLabel.setText(resourceMap.getString("jLabel12.text")); // NOI18N
        fromLabel.setName("fromLabel"); // NOI18N
        statusTabPanel.add(fromLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(334, 96, -1, -1));

        classLabel.setText(resourceMap.getString("jLabel13.text")); // NOI18N
        classLabel.setName("classLabel"); // NOI18N
        statusTabPanel.add(classLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(574, 100, -1, -1));

        methodValue.setText(resourceMap.getString("jLabel14.text")); // NOI18N
        methodValue.setName("methodValues"); // NOI18N
        statusTabPanel.add(methodValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 69, -1, -1));

        executionValue.setText(resourceMap.getString("jLabel15.text")); // NOI18N
        executionValue.setName("executionValue"); // NOI18N
        statusTabPanel.add(executionValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(482, 75, -1, -1));

        classValue.setText(resourceMap.getString("jLabel17.text")); // NOI18N
        classValue.setName("classValue"); // NOI18N
        statusTabPanel.add(classValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(482, 100, -1, -1));

        attributesValue.setText(resourceMap.getString("jLabel18.text")); // NOI18N
        attributesValue.setName("attributesValue"); // NOI18N
        statusTabPanel.add(attributesValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 136, 764, -1));

        usedLabel.setText(resourceMap.getString("jLabel19.text")); // NOI18N
        usedLabel.setName("usedLabel"); // NOI18N
        statusTabPanel.add(usedLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(334, 136, -1, -1));

        usedValue.setText(resourceMap.getString("jLabel20.text")); // NOI18N
        usedValue.setName("usedValue"); // NOI18N
        statusTabPanel.add(usedValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(482, 136, -1, -1));

        dataTransfered.setText(resourceMap.getString("jLabel21.text")); // NOI18N
        dataTransfered.setName("dataTransfered"); // NOI18N
        statusTabPanel.add(dataTransfered, new org.netbeans.lib.awtextra.AbsoluteConstraints(574, 136, -1, -1));

        usedInputValue.setText(resourceMap.getString("jLabel22.text")); // NOI18N
        usedInputValue.setName("usedInputValue"); // NOI18N
        statusTabPanel.add(usedInputValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 168, -1, -1));

        rulesLearntValue.setText(resourceMap.getString("jLabel23.text")); // NOI18N
        rulesLearntValue.setName("rulesLearntValue"); // NOI18N
        statusTabPanel.add(rulesLearntValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 200, -1, -1));

        rulesUnlearntValue.setText(resourceMap.getString("jLabel24.text")); // NOI18N
        rulesUnlearntValue.setName("rulesUnlearntValue"); // NOI18N
        statusTabPanel.add(rulesUnlearntValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(255, 232, -1, -1));

        inputsUsedLabel.setText(resourceMap.getString("jLabel25.text")); // NOI18N
        inputsUsedLabel.setName("inputsUsedLabel"); // NOI18N
        statusTabPanel.add(inputsUsedLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(352, 200, -1, -1));

        rulesLearntInputUsedValue.setText(resourceMap.getString("jLabel26.text")); // NOI18N
        rulesLearntInputUsedValue.setName("rulesLearntInputUsedValue"); // NOI18N
        statusTabPanel.add(rulesLearntInputUsedValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(498, 200, -1, -1));

        rulesUnlearntInputsLabel.setText(resourceMap.getString("jLabel27.text")); // NOI18N
        rulesUnlearntInputsLabel.setName("rulesUnlearntInputsLabel"); // NOI18N
        statusTabPanel.add(rulesUnlearntInputsLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(352, 232, -1, -1));

        rulesUnlearntInputsValue.setText(resourceMap.getString("jLabel28.text")); // NOI18N
        rulesUnlearntInputsValue.setName("rulesUnlearntInputsValue"); // NOI18N
        statusTabPanel.add(rulesUnlearntInputsValue, new org.netbeans.lib.awtextra.AbsoluteConstraints(498, 232, -1, -1));

        jTabbedPane1.addTab(resourceMap.getString("jPanel1.TabConstraints.tabTitle"), statusTabPanel); // NOI18N

        rulesTabPanel.setName("jPanel2"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        rulesTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null},
                {null, null, null},
                {null, null, null},
                {null, null, null}
            },
            new String [] {
                "Active", "Rule", "Class"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Boolean.class, java.lang.Object.class, java.lang.Object.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        rulesTable.setName("rulesTable"); // NOI18N
        rulesTable.setModel(ruleTableModel);
        ruleTableModel.addColumn("Active");
        ruleTableModel.addColumn("Rule");
        ruleTableModel.addColumn("Class");
        
        jScrollPane1.setViewportView(rulesTable);
        rulesTable.getColumnModel().getColumn(0).setHeaderValue(resourceMap.getString("jTable1.columnModel.title0")); // NOI18N
        rulesTable.getColumnModel().getColumn(1).setHeaderValue(resourceMap.getString("jTable1.columnModel.title1")); // NOI18N
        rulesTable.getColumnModel().getColumn(2).setHeaderValue(resourceMap.getString("jTable1.columnModel.title2")); // NOI18N
        rulesTable.getColumn("Active").setCellEditor(new DefaultCellEditor(checkBox2));

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(rulesTabPanel);
        rulesTabPanel.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(18, 18, 18)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 773, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 328, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(99, 99, 99))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel2.TabConstraints.tabTitle"), rulesTabPanel); // NOI18N

        profileTabPanel.setName("jPanel3"); // NOI18N

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        //
        //code modified by Aswathy Nair
        //
        autoClusterButton.setText("Auto Cluster"); // NOI18N
        autoClusterButton.setName("jButton11"); // NOI18N
        
        
        profileTable.setModel(profileTableModel);
        profileTableModel.addColumn("Profile No");
        profileTableModel.addColumn("Elapsed Time");
        profileTableModel.addColumn("Methods");
        profileTableModel.addColumn("Invocations");
        profileTableModel.addColumn("Attributes");
        profileTableModel.addColumn("Data, Kb");
        profileTableModel.addColumn("Class");
        profileTableModel.addColumn("Exclude");
        
        profileTable.setName("profileTable"); // NOI18N
        jScrollPane2.setViewportView(profileTable);
        profileTable.getColumnModel().getColumn(0).setHeaderValue(resourceMap.getString("jTable2.columnModel.title0")); // NOI18N
        profileTable.getColumnModel().getColumn(1).setHeaderValue("Elapsed Time"); // NOI18N
        profileTable.getColumnModel().getColumn(2).setHeaderValue(resourceMap.getString("jTable2.columnModel.title1")); // NOI18N
        profileTable.getColumnModel().getColumn(3).setHeaderValue("Invocations"); // NOI18N
        profileTable.getColumnModel().getColumn(4).setHeaderValue(resourceMap.getString("jTable2.columnModel.title3")); // NOI18N
        profileTable.getColumnModel().getColumn(5).setHeaderValue(resourceMap.getString("jTable2.columnModel.title4")); // NOI18N
        profileTable.getColumnModel().getColumn(6).setHeaderValue(resourceMap.getString("jTable2.columnModel.title5")); // NOI18N
        profileTable.getColumnModel().getColumn(7).setHeaderValue(resourceMap.getString("jTable2.columnModel.title6")); // NOI18N
        profileTable.getColumn("Exclude").setCellEditor(new DefaultCellEditor(excludeCheckBox));
        profileTable.getColumn("Class").setCellEditor(new DefaultCellEditor(combo));

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(profileTabPanel);
        profileTabPanel.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                //
                //code modified by Aswathy Nair
                //
                //.add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 781, Short.MAX_VALUE)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                   .add(autoClusterButton)
                   .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 781, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 330, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                //
                //code modified by Aswathy Nair
                //
                .add(autoClusterButton)
                .addContainerGap())
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel3.TabConstraints.tabTitle"), profileTabPanel); // NOI18N

        configTabPanel.setName("jPanel4"); // NOI18N

        jLabel16.setText(resourceMap.getString("jLabel16.text")); // NOI18N
        jLabel16.setName("jLabel16"); // NOI18N

        jScrollPane3.setName("jScrollPane3"); // NOI18N

        categoriesInputList.setName("categoriesInputList"); // NOI18N
        jScrollPane3.setViewportView(categoriesInputList);

        addConfigInputs.setText(resourceMap.getString("jButton6.text")); // NOI18N
        addConfigInputs.setName("jButton6"); // NOI18N

        removeConfigInputs.setText(resourceMap.getString("jButton7.text")); // NOI18N
        removeConfigInputs.setName("jButton7"); // NOI18N
        
        addButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            	addButtonMouseClicked(evt);
            }
          });
        deleteButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            	deleteButtonMouseClicked(evt);
            	
            }
         });
        
        //Code modified by Aswathy Nair
        //Adding auto cluster
        //
        autoClusterButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            	autoClusterButtonMouseClicked(evt);
            }
          });
      //Code modified by Aswathy Nair
        //Added logic for auto cluster
        //
        
        addButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            	addButtonMouseClicked(evt);
            }
          });
        /*combo.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                comboItemStateChanged(evt);
            }
        });
        combo.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	if("comboBoxChanged".equals(e.getActionCommand()) && !bNewClassIsBeingAdded)
    	    		doSomething();
    	    }
    	});
    	*/
        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator1.setName("jSeparator1"); // NOI18N

        packagesLabel.setText(resourceMap.getString("jLabel29.text")); // NOI18N
        packagesLabel.setName("packagesLabel"); // NOI18N

        configurationClassLabel.setText(resourceMap.getString("jLabel30.text")); // NOI18N
        configurationClassLabel.setName("jLabel30"); // NOI18N

        configurationMethodLabel.setText(resourceMap.getString("jLabel31.text")); // NOI18N
        configurationMethodLabel.setName("configurationMethodLabel"); // NOI18N

        configurationAttributesLabel.setText(resourceMap.getString("jLabel32.text")); // NOI18N
        configurationAttributesLabel.setName("configurationAttributesLabel"); // NOI18N

        configExcludeButton.setText(resourceMap.getString("jButton8.text")); // NOI18N
        configExcludeButton.setName("configExcludeButton"); // NOI18N

        configIncludeButton.setText(resourceMap.getString("jButton9.text")); // NOI18N
        configIncludeButton.setName("jButton9"); // NOI18N

        recommendationTextArea.setText(resourceMap.getString("jTextField1.text")); // NOI18N
        recommendationTextArea.setName("jTextField1"); // NOI18N

        configMethodTextArea.setText(resourceMap.getString("jTextField2.text")); // NOI18N
        configMethodTextArea.setName("jTextField2"); // NOI18N

        configMethodTextArea.setText(resourceMap.getString("jTextField3.text")); // NOI18N
        configMethodTextArea.setName("configMethodTextArea"); // NOI18N

        ConfigAttrTextArea.setText(resourceMap.getString("jTextField4.text")); // NOI18N
        ConfigAttrTextArea.setName("jTextField4"); // NOI18N

        jSeparator2.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jSeparator2.setName("jSeparator2"); // NOI18N

        classificationClassLabel.setText(resourceMap.getString("jLabel33.text")); // NOI18N
        classificationClassLabel.setName("classificationClassLabel"); // NOI18N

        jScrollPane4.setName("jScrollPane4"); // NOI18N

        configClassificationClassList.setName("configClassificationClassList"); // NOI18N
        jScrollPane4.setViewportView(configClassificationClassList);

        deleteButton.setText("Remove Class"); // NOI18N
        deleteButton.setName("deleteButton"); // NOI18N
        deleteButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            	deleteButtonMouseClicked(evt);
            }
        });
        addButton.setText("Add Class"); // NOI18N
        addButton.setName("jButton11"); // NOI18N

        newCategoryLabel.setText(resourceMap.getString("jLabel34.text")); // NOI18N
        newCategoryLabel.setName("newCategoryLabel"); // NOI18N

        newClassTextArea.setText(resourceMap.getString("jTextField5.text")); // NOI18N
        newClassTextArea.setName("jTextField5"); // NOI18N

        org.jdesktop.layout.GroupLayout jPanel4Layout = new org.jdesktop.layout.GroupLayout(configTabPanel);
        configTabPanel.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel4Layout.createSequentialGroup()
                .add(39, 39, 39)
                .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(jPanel4Layout.createSequentialGroup()
                        .add(addConfigInputs)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(removeConfigInputs))
                    .add(jLabel16)
                    .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 210, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(38, 38, 38)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel4Layout.createSequentialGroup()
                        .add(41, 41, 41)
                        .add(configExcludeButton)
                        .add(31, 31, 31)
                        .add(configIncludeButton))
                    .add(jPanel4Layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(configurationAttributesLabel)
                            .add(configurationClassLabel)
                            .add(packagesLabel)
                            .add(configurationMethodLabel))
                        .add(29, 29, 29)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(recommendationTextArea)
                            .add(ConfigAttrTextArea)
                            .add(configMethodTextArea)
                            .add(configMethodTextArea, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 136, Short.MAX_VALUE))))
                .add(18, 18, 18)
                .add(jSeparator2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel4Layout.createSequentialGroup()
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel4Layout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 54, Short.MAX_VALUE)
                                .add(newCategoryLabel))
                            .add(jPanel4Layout.createSequentialGroup()
                                .add(32, 32, 32)
                                .add(deleteButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)))
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel4Layout.createSequentialGroup()
                                .add(11, 11, 11)
                                .add(newClassTextArea, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 105, Short.MAX_VALUE)
                                .add(177, 177, 177))
                            .add(jPanel4Layout.createSequentialGroup()
                                .add(34, 34, 34)
                                .add(addButton)
                                .addContainerGap())))
                    .add(jPanel4Layout.createSequentialGroup()
                        .add(18, 18, 18)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jScrollPane4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 219, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(classificationClassLabel))
                        .add(179, 179, 179))))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel4Layout.createSequentialGroup()
                .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jPanel4Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 330, Short.MAX_VALUE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel4Layout.createSequentialGroup()
                        .add(40, 40, 40)
                        .add(jLabel16)
                        .add(18, 18, 18)
                        .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 180, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(18, 18, 18)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(addConfigInputs)
                            .add(removeConfigInputs)))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel4Layout.createSequentialGroup()
                        .add(38, 38, 38)
                        .add(classificationClassLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jScrollPane4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 186, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(18, 18, 18)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(deleteButton)
                            .add(addButton))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(newCategoryLabel)
                            .add(newClassTextArea, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel4Layout.createSequentialGroup()
                        .add(40, 40, 40)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(packagesLabel)
                            .add(recommendationTextArea, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(26, 26, 26)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(configurationClassLabel)
                            .add(configMethodTextArea, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(34, 34, 34)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(configurationMethodLabel)
                            .add(configMethodTextArea, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(32, 32, 32)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(configurationAttributesLabel)
                            .add(ConfigAttrTextArea, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 60, Short.MAX_VALUE)
                        .add(jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(configExcludeButton)
                            .add(configIncludeButton))
                        .add(46, 46, 46)))
                .addContainerGap())
            .add(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .add(jSeparator2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 337, Short.MAX_VALUE)
                .add(4, 4, 4))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel4.TabConstraints.tabTitle"), configTabPanel); // NOI18N

        analysisTabPanel.setName("jPanel5"); // NOI18N

        org.jdesktop.layout.GroupLayout jPanel5Layout = new org.jdesktop.layout.GroupLayout(analysisTabPanel);
        analysisTabPanel.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 801, Short.MAX_VALUE)
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 352, Short.MAX_VALUE)
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel5.TabConstraints.tabTitle"), analysisTabPanel); // NOI18N

        bottleneckTabPanel.setName("bottleneckTabPanel"); // NOI18N

        jScrollPane5.setName("jScrollPane5"); // NOI18N

        bottleneckTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null}
            },
            new String [] {
                "Class", "Method", "Weight"
            }
        ) {
        	Class[] types = new Class [] {
        			java.lang.Boolean.class, java.lang.Object.class, java.lang.Object.class	
        	};
        	
        	public Class getColumnClass(int columnIndex){
        		return types[columnIndex];
        	}
        } );
        bottleneckTable.setName("bottleneckTable"); 
        bottleneckTable.setModel(bottleneckTableModel);
        bottleneckTableModel.addColumn("Class");
        bottleneckTableModel.addColumn("Method");
        bottleneckTableModel.addColumn("Weight");
        
        // NOI18N
        jScrollPane5.setViewportView(bottleneckTable);
        bottleneckTable.getColumnModel().getColumn(0).setHeaderValue(resourceMap.getString("jTable3.columnModel.title0")); // NOI18N
        bottleneckTable.getColumnModel().getColumn(1).setHeaderValue(resourceMap.getString("jTable3.columnModel.title1")); // NOI18N
        bottleneckTable.getColumnModel().getColumn(2).setHeaderValue(resourceMap.getString("jTable3.columnModel.title2")); // NOI18N
       /* bottleneckTable.getColumnModel().getColumn(3).setHeaderValue(resourceMap.getString("jTable3.columnModel.title3")); // NOI18N
        bottleneckTable.getColumnModel().getColumn(4).setHeaderValue(resourceMap.getString("jTable3.columnModel.title4")); // NOI18N
        bottleneckTable.getColumnModel().getColumn(5).setHeaderValue(resourceMap.getString("jTable3.columnModel.title5")); // NOI18N
        bottleneckTable.getColumnModel().getColumn(6).setHeaderValue(resourceMap.getString("jTable3.columnModel.title6")); // NOI18N
        bottleneckTable.getColumnModel().getColumn(7).setHeaderValue(resourceMap.getString("jTable3.columnModel.title7")); // NOI18N
*/
        org.jdesktop.layout.GroupLayout jPanel6Layout = new org.jdesktop.layout.GroupLayout(bottleneckTabPanel);
        bottleneckTabPanel.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 781, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel6Layout.createSequentialGroup()
                .addContainerGap(12, Short.MAX_VALUE)
                .add(jScrollPane5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 329, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel6.TabConstraints.tabTitle"), bottleneckTabPanel); // NOI18N

        recommendationTabPanel.setName("jPanel7"); // NOI18N

        jScrollPane6.setName("jScrollPane6"); // NOI18N

        configPackagesText.setColumns(20);
        configPackagesText.setRows(5);
        configPackagesText.setName("configPackagesText"); // NOI18N
        jScrollPane6.setViewportView(configPackagesText);

        org.jdesktop.layout.GroupLayout jPanel7Layout = new org.jdesktop.layout.GroupLayout(recommendationTabPanel);
        recommendationTabPanel.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 781, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel7Layout.createSequentialGroup()
                .add(19, 19, 19)
                .add(jScrollPane6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 322, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel7.TabConstraints.tabTitle"), recommendationTabPanel); // NOI18N

        org.jdesktop.layout.GroupLayout mainPanelLayout = new org.jdesktop.layout.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 806, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(53, Short.MAX_VALUE))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 380, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(22, Short.MAX_VALUE))
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(forepost.FOREPOSTApp.class).getContext().getActionMap(FOREPOSTView.class, this);

        persistMenu.setText("Persist"); // NOI18N
        persistMenu.setName("persistMenu"); // NOI18N

        persistARFFMenu.setText("ARFF"); // NOI18N
        persistARFFMenu.setName("persistARFFMenu"); // NOI18N
        persistARFFMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	testStartMenuMousePressed(evt);
            }
        });
        persistMenu.add(persistARFFMenu);
        fileMenu.add(persistMenu);

        persistCallgraphMenu.setText("Callgraph"); // NOI18N
        persistCallgraphMenu.setName("persistCallgraphMenu"); // NOI18N
        persistMenu.add(persistCallgraphMenu);
     
        fileLoadMenu.setText(resourceMap.getString("jMenu6.text")); // NOI18N
        fileLoadMenu.setName("fileLoadMenu"); // NOI18N

        loadProfileStatsMenu.setText(resourceMap.getString("jMenuItem11.text")); // NOI18N
        loadProfileStatsMenu.setName("loadProfileStatsMenu"); // NOI18N
        fileLoadMenu.add(loadProfileStatsMenu);

        loadDataAttributesMenu.setText(resourceMap.getString("jMenuItem13.text")); // NOI18N
        loadDataAttributesMenu.setName("loadDataAttributesMenu"); // NOI18N
        fileLoadMenu.add(loadDataAttributesMenu);

        loadInputDataMenu.setText(resourceMap.getString("jMenuItem14.text")); // NOI18N
        loadInputDataMenu.setName("loadInputDataMenu"); // NOI18N
        fileLoadMenu.add(loadInputDataMenu);

 

        fileMenu.add(fileLoadMenu);

        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);
        
        menuBar.add(fileMenu);

        controlMenu.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        controlMenu.setName("controlMenu"); // NOI18N

//        Control/Analyzer menu
        analyzerMenu.setText("Analyzer");
        analyzerMenu.setName("analyzerMenu"); // NOI18N

        analyzerStartMenu.setText("Start"); // NOI18N
        analyzerStartMenu.setName("analyzerStartMenu"); // NOI18N
        analyzerStartMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            }
            
            public void mousePressed(java.awt.event.MouseEvent evt) {
                try {
//					dataStartMenuMousePressed(evt);
//					dataPauseMenuMousePressed(evt);
//					dataResumeMenuMousePressed(evt);
//					testPauseMenuMousePressed(evt);
//					testStartMenuMousePressed(evt);
//					testResetMenuMousePressed(evt);
//					testResumeMenuMousePressed(evt);
//					wekaPauseMenuMousePressed(evt);
//					wekaResumeMenuMousePressed(evt);
//					rulesRelearnMenuMousePressed(evt);
//					rulesUpdateMenuMousePressed(evt);
//					threadStopMenuMousePressed(evt);
//					threadSuspendMenuMousePressed(evt);
//					profileManualClusterMenuMousePressed(evt);
//					profileRelearnMenuMousePressed(evt);
//					bottleneckCallGraphMenuMousePressed(evt);
					startAnalyzer(evt);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
                
                
            }
        });
        analyzerStartMenu.addMenuKeyListener(new javax.swing.event.MenuKeyListener() {
            public void menuKeyPressed(javax.swing.event.MenuKeyEvent evt) {
                
            }
            public void menuKeyReleased(javax.swing.event.MenuKeyEvent evt) {
            }
            public void menuKeyTyped(javax.swing.event.MenuKeyEvent evt) {
            }
        });
        analyzerMenu.add(analyzerStartMenu);

        analyzerPauseMenu.setText("Pause"); // NOI18N
        analyzerPauseMenu.setName("analyzerPauseMenu"); // NOI18N
        analyzerPauseMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	try {
					dataPauseMenuMousePressed(evt);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
            }
        });
        analyzerPauseMenu.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
             
            }
        });
        analyzerMenu.add(analyzerPauseMenu);

        analyzerResumeMenu.setText("Resume"); // NOI18N
        analyzerResumeMenu.setName("analyzerResumeMenu"); // NOI18N
        analyzerResumeMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	try {
					analyzerResumeMenuMousePressed(evt);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
            }
        });
        analyzerResumeMenu.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
             
            }
        });        
        analyzerMenu.add(analyzerResumeMenu);

        controlMenu.add(analyzerMenu);


//        controlMenu.add(persistMenu);
/*
        wekaMenu.setText(resourceMap.getString("jMenu4.text")); // NOI18N
        wekaMenu.setName("wekaMenu"); // NOI18N

        wekaStartMenu.setText(resourceMap.getString("jMenuItem10.text")); // NOI18N
        wekaStartMenu.setName("wekaStartMenu"); // NOI18N
        wekaStartMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	wekaStartMenuMousePressed(evt);
            }
        });
        wekaStartMenu.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
              
            }
        });
        wekaMenu.add(wekaStartMenu);

        wekaPauseMenu.setText(resourceMap.getString("jMenuItem6.text")); // NOI18N
        wekaPauseMenu.setName("wekaPauseMenu"); // NOI18N
        wekaMenu.add(wekaPauseMenu);

        wekaResumeMenu.setText(resourceMap.getString("jMenuItem7.text")); // NOI18N
        wekaResumeMenu.setName("wekaResumeMenu"); // NOI18N
        wekaMenu.add(wekaResumeMenu);

        controlMenu.add( wekaMenu);
*/
        rulesMenu.setText(resourceMap.getString("jMenu8.text")); // NOI18N
        rulesMenu.setName("rulesMenu"); // NOI18N

        rulesRelearnMenu.setText(resourceMap.getString("jMenuItem19.text")); // NOI18N
        rulesRelearnMenu.setName("rulesRelearnMenu"); // NOI18N
        rulesRelearnMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	try {
            		rulesRelearnMenuMousePressed(evt);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
            }
        });
        analyzerResumeMenu.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
             
            }
        });
        
        rulesMenu.add(rulesRelearnMenu);

        rulesUpdateMenu.setText("Decision Tree"); // NOI18N
        rulesUpdateMenu.setName("Visualize Decision Tree"); // NOI18N
        rulesUpdateMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	try {
            		rulesUpdateMenuMousePressed(evt);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
            }
        });
        
        rulesMenu.add(rulesUpdateMenu);
        controlMenu.add(rulesMenu);


        bottleneckMenu.setText(resourceMap.getString("jMenu11.text")); // NOI18N
        bottleneckMenu.setName("bottleneckMenu"); // NOI18N

        bottleneckExcludeMenu.setText(resourceMap.getString("jMenuItem22.text")); // NOI18N
        bottleneckExcludeMenu.setName("bottleneckExcludeMenu"); // NOI18N
        bottleneckMenu.add(bottleneckExcludeMenu);

        bottleneckViewMenu.setText(resourceMap.getString("jMenuItem23.text")); // NOI18N
        bottleneckViewMenu.setName("bottleneckViewMenu"); // NOI18N
        bottleneckMenu.add(bottleneckViewMenu);
       
// QI
        bottleneckViewMenu.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
            }
            
            public void mousePressed(java.awt.event.MouseEvent evt) {
                try {
                	bottleneckViewMenuMousePressed(evt);
                } catch (Exception e) {
					e.printStackTrace();
					
				}               
            }
        });
// QI

        bottleneckCallGraphMenu.setText(resourceMap.getString("jMenuItem24.text")); // NOI18N
        bottleneckCallGraphMenu.setName("bottleneckCallGraphMenu"); // NOI18N
        bottleneckMenu.add(bottleneckCallGraphMenu);

        controlMenu.add(bottleneckMenu);

        menuBar.add(controlMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        org.jdesktop.layout.GroupLayout statusPanelLayout = new org.jdesktop.layout.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(statusPanelSeparator, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 869, Short.MAX_VALUE)
            .add(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(statusMessageLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 699, Short.MAX_VALUE)
                .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(statusPanelLayout.createSequentialGroup()
                .add(statusPanelSeparator, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(statusPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(statusMessageLabel)
                    .add(statusAnimationLabel)
                    .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(3, 3, 3))
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents


////////////////////////////////////////////////////////////////////////////////
  private void dataStartMenuMousePressed(java.awt.event.MouseEvent evt) {                                        
        // TODO add your handling code here:
        //Data Menu -> Start
//        System.out.print("Data.. Mouse Pressed..Menu key is pressed\n"); hide_focus
        DataThread d = new DataThread();
        new Thread(d).start();
//        System.out.print("Reached end of Data\n"); hide_focus
    } 

///////////////////////////////////////////////////////////////////////

 private void dataPauseMenuMousePressed(java.awt.event.MouseEvent evt) throws Exception 
 {                                        
		if( analyzer == null) {
			FPUtility.getLogger().log(Level.SEVERE, "Analyzer object is null");
			return;
		}
		analyzer.Suspend();
}

 private void analyzerResumeMenuMousePressed(java.awt.event.MouseEvent evt) throws Exception 
 {                                        
		if( analyzer == null) {
			FPUtility.getLogger().log(Level.SEVERE, "Analyzer object is null");
			return;
		}
		analyzer.Resume();
    }
 
////////////////////////////////////////////////////////////////////////
 private void dataResumeMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //Data-Resume
    }
//////////////////////////////////////////////////////////////////////////
  private void testPauseMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //Test - Pause
    }
///////////////////////////////////////////////////////////////////////////
  private void testStartMenuMousePressed(java.awt.event.MouseEvent evt) {                                        
        // TODO add your handling code here:
	//Test Start
//        System.out.print("Test.. Mouse Pressed..Menu key is pressed\n"); hide_focus
        TestThread t = new TestThread();
        new Thread(t).start();
//        System.out.print("Reached end of Test\n"); hide_focus
    }      
//////////////////////////////////////////////////////////////////////////////
 private void testResetMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //Test - Reset
    }
////////////////////////////////////////////////////////////////////////////////
 private void testResumeMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //Test - Resume
    }
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
  private void wekaPauseMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //WEkA- Pause
    }
////////////////////////////////////////////////////////////////////////////////
    private void wekaResumeMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //WEKA - Resume
    }
/////////////////////////////////////////////////////////////////////////////////
 private void rulesRelearnMenuMousePressed(java.awt.event.MouseEvent evt) {
	 javax.swing.table.DefaultTableModel ruleTableModel =  getRuleTableModel();
	 if( ruleTableModel != null ){
		 while( ruleTableModel.getDataVector().size() > 0 ) ruleTableModel.removeRow(0);
	 }
	 
	 int numProfiles = profileTableModel.getDataVector().size();
	 FPUtility.RuleLog("Enumerating classes for " + String.valueOf(numProfiles) + " profiles" );
//	 logger.info("start learn rules!"); //QI
	 for( int i = 0; i < numProfiles; i++ ){
		 try {
			 FPUtility.RuleLog("Profile " + String.valueOf(i+1) + " : " + "time = " + String.valueOf(profileTableModel.getValueAt(i, 1)) + 
					 "\nmethods = " + String.valueOf(profileTableModel.getValueAt(i, 2))  + 
					 "\ninvoked = " + String.valueOf(profileTableModel.getValueAt(i, 3))  +
					 "\nattributes = " + String.valueOf(profileTableModel.getValueAt(i, 4)) +
					 "\nKb = " + String.valueOf(profileTableModel.getValueAt(i, 5))  +
					 "\nclass = " + String.valueOf(profileTableModel.getValueAt(i, 6)) );
			//this.getAnalyzer().SetProfileClass(String.valueOf(profileTableModel.getValueAt(i, 6)), i);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		 System.out.println(profileTableModel.getValueAt(i, 6));
	 }
	 
	 try {
		 if(autocluster) 
			 analyzer.Analyze(true, numProfiles, FPUtility.getARFFRecalculateFlag());
		 else
			 analyzer.Analyze(false, numProfiles, FPUtility.getARFFRecalculateFlag());
		} catch (Exception e) {
			e.printStackTrace();
		}
}
///////////////////////////////////////////////////////////////////////////////
  private void rulesUpdateMenuMousePressed(java.awt.event.MouseEvent evt) {
	  try
	  {
		  this.getAnalyzer().GetInputParamDecisionTree();
	} catch (Exception e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	  }
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
 private void threadStopMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //Thread-Stop
    }
////////////////////////////////////////////////////////////////////////////////
 private void threadSuspendMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //Thread-Suspend
    }
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
 private void profileManualClusterMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //profile-manual cluster
    }
/////////////////////////////////////////////////////////////////////////////////
 private void profileRelearnMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //profile - relearn
    }
///////////////////////////////////////////////////////////////////////////////////
 
////////////////////////////////////////////////////////////////////////////////////
  private void bottleneckViewMenuMousePressed(java.awt.event.MouseEvent evt) throws Exception {
	  
		 javax.swing.table.DefaultTableModel bottleneckTableModel =  getBottleNeckModel();
		 if( bottleneckTableModel != null ){
			 while( bottleneckTableModel.getDataVector().size() > 0 ) bottleneckTableModel.removeRow(0);
		 }
		 
		if (analyzer == null) {
			FPUtility.getLogger().log(Level.SEVERE, "Analyzer object is null");
			return;
		}
//		new Thread().analyzeICA();
//		Thread thread = new Thread(analyzer);
//		thread.analyzeICA();

				analyzer.analyzeICA();
//		new Thread(analyzerICA).start();
		
        //Bottleneck-view
    }
///////////////////////////////////////////////////////////////////////////////////
 private void bottleneckCallGraphMenuMousePressed(java.awt.event.MouseEvent evt) {
        // TODO add your handling code here:
        //bottleneck - call graph
//	    System.out.print("Bottleneck call graph\n"); hide_focus
    }
////////////////////////////////////////////////////////////////////////////////////
 private void startAnalyzer(java.awt.event.MouseEvent evt) throws Exception 
 {
		if( analyzer == null) {
			FPUtility.getLogger().log(Level.SEVERE, "Analyzer object is null");
			return;
		}
		new Thread(analyzer).start();
//        System.out.println("Started the analyzer"); hide_focus
    }//GEN-LAST:event_jMenuItem10MousePressed
 ////////////////////////////////////////////////////////////////////////////////
 private void addButtonMouseClicked(java.awt.event.MouseEvent evt) {
     // TODO add your handling code here:
     //add button clicked to add class
	 LinkedList classList = new LinkedList();
	 	
	 try{
		   bNewClassIsBeingAdded = true;
		   classList = this.analyzer.getClasses();
		   classList.add(newClassTextArea.getText());
		   this.analyzer.setClasses(classList);
		   this.combo.addItem(newClassTextArea.getText());
		   bNewClassIsBeingAdded = false;
//		   this.combo.removeAllItems();
//	       for(int i=0;i<classList.size();i++){
//	      	 this.combo.addItem(classList.get(i));
//	      	 }
	       
	       
	 }catch( Exception e )
	 {
		 e.printStackTrace();
		 System.out.print("Could not set class");
	 }
	 this.configClassificationClassList.setListData(this.analyzer.Classes.toArray());
	
 }
 
 private void deleteButtonMouseClicked(java.awt.event.MouseEvent evt) {
     // TODO add your handling code here:
     //delete button clicked
	 bNewClassIsBeingAdded = true;
	 int ind = this.configClassificationClassList.getSelectedIndex();
	 if( ind < 0 )return;
     this.analyzer.Classes.remove(ind);
     this.configClassificationClassList.setListData(this.analyzer.Classes.toArray());
     this.combo.removeAllItems();
     
//     int profileID=this.profileTable.getSelectedRow()+1;
     try{
//     this.getAnalyzer().SetProfileClass("Unassigned", profileID);     
     for(int i=0;i<this.analyzer.getClasses().size();i++){
      	 this.combo.addItem(this.analyzer.getClasses().get(i));
      	 
      	 }
     }catch( Exception e )
	 {
		 e.printStackTrace();
		 System.out.print("Could not set class");
	 }
     bNewClassIsBeingAdded = false;     
 }
//////////////////////////////////////////////////////////////////////////////////
/* public void comboItemStateChanged(java.awt.event.ItemEvent evt) {
     // TODO add your handling code here:
	  
	 try{
	 String Class = this.combo.getSelectedItem().toString();
	 int profileID=this.profileTable.getSelectedRow()+1;
	 //System.out.print(Class+" "+profileID);
	 this.getAnalyzer().SetProfileClass(Class, profileID);
	 }
	 catch( Exception e )
	 {
		 e.printStackTrace();
		 System.out.print("Could not set class");
	 }
	 
 }
 */
 public void doSomething()
 {
	 String Class=null;
	 int profileID=0;
	 try{

		 profileID=this.profileTable.getSelectedRow();
		 if( profileID < 0 )
			 return;
		 
		 if(combo.getSelectedItem()==null)
		 {
			 Class = FPUtility.UNASSIGNEDCLASS;
		 }
		 else
		 {
			 Class = combo.getSelectedItem().toString();
		 }
//		 System.out.println(Class+" "+profileID); hide_focus
		 this.getAnalyzer().SetProfileClass(Class, profileID);
		 }
		 catch( Exception e )
		 {
			 System.out.println("Exception for setting the class "+Class+" for profile "+profileID);
//			 e.printStackTrace();
//			 System.out.print("Could not set class");
		 } 
 
 }

 

 /////////////////////////////////////////////////////////////////////////////////
    public void setAnalyzer(ProfileAnalyzer analyzer) {
		this.analyzer = analyzer;
	}

 

	public ProfileAnalyzer getAnalyzer() {
		return analyzer;
	}

	/////////////////code modified by Aswathy Nair
	private void autoClusterButtonMouseClicked(java.awt.event.MouseEvent evt) {
//		logger.info("start to cluster the profiles based on the elapsed time!"); //QI
		try {
			autocluster = true;
			analyzer.getClasses().clear();
			int total = profileTableModel.getDataVector().size();
			LinkedList<String> classes = new LinkedList<String>();
			classes.add("Good");
			classes.add("Bad");
			//classes.add("Unassigned");
			analyzer.setClasses(classes);
			this.combo.removeAllItems();
			for(int i=0;i<this.analyzer.getClasses().size();i++){
		      	 this.combo.addItem(this.analyzer.getClasses().get(i));
		      	 
		    }
			double averageTime = analyzer.GetElapsedTime() / total ;
			for(int i = 0; i < total; i++){
			profileTableModel.setValueAt(analyzer.AssignClasses(averageTime, i), i, 6);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	
	
	// Variables declaration - do not modify//GEN-BEGIN:variables
    public  javax.swing.JButton deleteButton;
    public javax.swing.JButton addButton;
    public javax.swing.JButton addConfigInputs;
    public javax.swing.JButton removeConfigInputs;
    public javax.swing.JButton configExcludeButton;
    public javax.swing.JButton configIncludeButton ;
    public javax.swing.JButton autoClusterButton ;
    public javax.swing.JLabel statusLabel;
    public javax.swing.JLabel executedLabel;
    public javax.swing.JLabel timeLabel;
    public javax.swing.JLabel fromLabel;
    public javax.swing.JLabel classLabel;
    public javax.swing.JLabel methodValue;
    public javax.swing.JLabel executionValue;
    public javax.swing.JLabel jLabel16;
    public javax.swing.JLabel classValue;
    public javax.swing.JLabel attributesValue;
    public javax.swing.JLabel usedLabel;
    public javax.swing.JLabel runsLabel;
    public javax.swing.JLabel usedValue;
    public javax.swing.JLabel dataTransfered;
    public javax.swing.JLabel usedInputValue;
    public javax.swing.JLabel rulesLearntValue;
    public javax.swing.JLabel rulesUnlearntValue;
    public javax.swing.JLabel inputsUsedLabel;
    public javax.swing.JLabel rulesLearntInputUsedValue;
    public javax.swing.JLabel rulesUnlearntInputsLabel;
    public javax.swing.JLabel rulesUnlearntInputsValue;
    public javax.swing.JLabel packagesLabel;
    public javax.swing.JLabel methodLabel;
    public javax.swing.JLabel configurationClassLabel;
    public javax.swing.JLabel configurationMethodLabel;
    public javax.swing.JLabel configurationAttributesLabel;
    public javax.swing.JLabel classificationClassLabel;
    public javax.swing.JLabel newCategoryLabel;
    public javax.swing.JLabel attributeLabel;
    public javax.swing.JLabel unusedInputLabel;
    public javax.swing.JLabel rulesLearntLabel;
    public javax.swing.JLabel rulesUnlearntLabel;
    public javax.swing.JLabel statusValue;
    public javax.swing.JLabel runsValue;
    public javax.swing.JList categoriesInputList;
    public javax.swing.JList configClassificationClassList;
    public javax.swing.JMenu controlMenu;
    public javax.swing.JMenu profileMenu;
    public javax.swing.JMenu bottleneckMenu;
    public javax.swing.JMenu analyzerMenu;
    public javax.swing.JMenu persistMenu;
    public javax.swing.JMenu wekaMenu;
    public javax.swing.JMenu actionMenu;
    public javax.swing.JMenu fileLoadMenu;
    public javax.swing.JMenu rulesMenu;
    public javax.swing.JMenu threadMenu;
    public javax.swing.JMenuItem analyzerPauseMenu;
    public javax.swing.JMenuItem wekaStartMenu;
    public javax.swing.JMenuItem loadProfileStatsMenu;
    public javax.swing.JMenuItem profileAutoClusterMenu;
    public javax.swing.JMenuItem loadDataAttributesMenu;
    public javax.swing.JMenuItem loadInputDataMenu;
    public javax.swing.JMenuItem profileManualClusterMenu;
    public javax.swing.JMenuItem threadStartMenu;
    public javax.swing.JMenuItem threadStopMenu;
    public javax.swing.JMenuItem rulesRelearnMenu;
    public javax.swing.JMenuItem analyzerResumeMenu;
    public javax.swing.JMenuItem rulesUpdateMenu;
    public javax.swing.JMenuItem threadSuspendMenu;
    public javax.swing.JMenuItem bottleneckExcludeMenu;
    public javax.swing.JMenuItem bottleneckViewMenu;
    public javax.swing.JMenuItem bottleneckCallGraphMenu;
    public javax.swing.JMenuItem persistCallgraphMenu;
    public javax.swing.JMenuItem persistLearnMenu;
    public javax.swing.JMenuItem persistResultMenu;
    public javax.swing.JMenuItem wekaPauseMenu;
    public javax.swing.JMenuItem wekaResumeMenu;
    public javax.swing.JMenuItem analyzerStartMenu;
    public javax.swing.JMenuItem persistARFFMenu;
    public javax.swing.JPanel statusTabPanel;
    public javax.swing.JPanel rulesTabPanel;
    public javax.swing.JPanel profileTabPanel;
    public javax.swing.JPanel configTabPanel;
    public javax.swing.JPanel analysisTabPanel;
    public javax.swing.JPanel bottleneckTabPanel;
    public javax.swing.JPanel recommendationTabPanel;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JScrollPane jScrollPane6;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JTabbedPane jTabbedPane1;
    public javax.swing.JTable rulesTable;
    public javax.swing.JTable profileTable;
    public javax.swing.JTable bottleneckTable;
    public javax.swing.JTextArea configPackagesText;
    public javax.swing.JTextField recommendationTextArea;
    public javax.swing.JTextField configClassTextArea;
    public javax.swing.JTextField configMethodTextArea;
    public javax.swing.JTextField ConfigAttrTextArea;
    public javax.swing.JTextField newClassTextArea;
    public javax.swing.JPanel mainPanel;
    public javax.swing.JMenuBar menuBar;
    public javax.swing.JProgressBar progressBar;
    public javax.swing.JLabel statusAnimationLabel;
    public javax.swing.JLabel statusMessageLabel;
    public javax.swing.JPanel statusPanel;
    public ClassComboBox combo;
    
    
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;


}
