package forepost;
import java.io.*;
import java.util.*;
import java.text.*;
import java.util.logging.*;

public class FPUtility {
	public static final String UNASSIGNEDCLASS = "Unassigned";
	private static String LogFile = null;
	private static Logger logger = null;//Logger.getLogger("FOREPOST");
	private final static Properties properties = new Properties();
	private static Writer ruleIntoLogFile = null;
	
	public static void startRuleInfoLogging(boolean overwrite) throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String fileName = properties.getProperty("RULE_Info_Log_Filename");
			File file = new File( getResultsDirectory() + fileName);
			ruleIntoLogFile = new BufferedWriter(new FileWriter(file, !overwrite));
			DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			Date date = new Date();
			ruleIntoLogFile.write( "Starting rule info log: " + dateFormat.format(date) + "\n" );
			ruleIntoLogFile.flush();
			ruleIntoLogFile.close();
			return;
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to a rule log file");
	}

	public static void RuleLog( String logString )
	{
		if( logString == null)
			return;

		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String fileName = properties.getProperty("RULE_Info_Log_Filename");
			File file = new File( getResultsDirectory() + fileName);
			ruleIntoLogFile = new BufferedWriter(new FileWriter(file, true));
			DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			Date date = new Date();
			ruleIntoLogFile.write( "New record: " + dateFormat.format(date) + "\n" );
			ruleIntoLogFile.write( logString + "\n" );
			ruleIntoLogFile.flush();
			ruleIntoLogFile.close();
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static Logger getLogger() throws Exception 
	{
		try {
			if( logger == null ){
				logger = Logger.getLogger("FOREPOST");
				logger.addHandler(new FileHandler(getLogFile()));
			    logger.setLevel(Level.ALL);
			}
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new Exception(e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new Exception(e);
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception(e);
		}
		if( logger == null ) throw new Exception("logger object is null");
		return( logger );
	}
	
	public static String getLogFile() throws Exception
	{
		if(LogFile == null){
			try {
				properties.loadFromXML(new FileInputStream("forepost.properties"));
				LogFile = properties.getProperty("logfile");
			} catch (InvalidPropertiesFormatException e) {
				e.printStackTrace();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if(LogFile == null) throw new Exception("unknown name for a log file");
		return( LogFile );
	}

	public static boolean getARFFRecalculateFlag() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String rec = properties.getProperty("ARFF_Recalculate_Flag");
			boolean recFlag = Boolean.parseBoolean(rec);
			return( recFlag );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an screen input values param file");
		
	}

	public static String getScreenInputValuesFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String fname = properties.getProperty("FirstLoop_Xml_file_name");
			if( fname == null )fname = "FP_FirstLoop.xml";
			return( getResultsDirectory() + fname );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an screen input values param file");
	}
	
	public static String getScreenInputACSTValuesFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String fname = properties.getProperty("ACST_file_name");
			if( fname == null )fname = "FP_ACST.txt";
			return( getResultsDirectory() + fname );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an screen input values param file");
	}

	public static String getScreenInputCSVValuesFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String fname = properties.getProperty("CSV_file_name");
			if( fname == null )fname = "FP_Params.csv";
			return( getResultsDirectory() + fname );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an screen input values param file");
	}
	
	
	public static String getRepository() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			return( checkDirectoryName(properties.getProperty("repository")) );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to a repository");
	}

	public static String getResultsDirectory() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String dir = checkDirectoryName(properties.getProperty("resultdir"));
			boolean exists = (new File(dir)).exists();
			if(!exists){
				if(!(new File(dir)).mkdirs()){
					throw new Exception( "Cannot create a directory for results" );
				}
			}
			return( dir );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the results path");
	}

	public static String getMethodExecTimesFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			return( getResultsDirectory() + properties.getProperty("methodexectimes_file_name") );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to the method exec times file");
	}
	
	
	public static String getARFFInputParamsFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			return( getResultsDirectory() + properties.getProperty("ARFF_input_params_file_name") );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an ARFF input param file");
	}

	public static String getExclusionFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			String fname = properties.getProperty("Input_Exclusion_INI_File");
			if( fname == null )throw new Exception("no input exclusion file");
			return( fname );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an INI input exclusion file");
	}
	
	public static String getARFFDbDataFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			return( getResultsDirectory() + properties.getProperty("ARFF_dbdata_file_name") );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an ARFF dbdata file");
	}
	
	public static String getARFFClusterProfileFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			return( getResultsDirectory() + properties.getProperty("ARFF_clusterprofiles_file_name") );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an ARFF profile clustering file");
	}

	public static String getFullProfileDataFile() throws Exception
	{
		try {
			properties.loadFromXML(new FileInputStream("forepost.properties"));
			return( getResultsDirectory() + properties.getProperty("fulldump") );
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Exception("Cannot obtain the path to an ARFF profile clustering file");
	}
	
	
	private static String checkDirectoryName(String dir){
		if( dir.charAt(dir.length()-1) != File.separatorChar )
			return( dir + File.separatorChar );
		return( dir );
	}
}
