package forepost;
import java.io.File;
import forepost.utilities.*;
import java.util.logging.Level;
import java.util.*;

import org.apache.log4j.Logger;

import com.accenture.techlabs.assist.profile.ProfRepository;

import weka.classifiers.rules.JRip;
import weka.core.*;
import weka.core.converters.ArffLoader;

public class ForePostRuleClassifier extends JRip {
	// QI
	private static final Logger logger = Logger.getLogger(ProfRepository.class);
	protected Instances data = null;
	protected Attribute consequent = null;
	protected FastVector ruleset = null;
	private String ArrowToken = " => ";
	private String AndToken = " and ";
	private String LEQToken = " <= ";
	private String GEQToken = " >= ";
	private String EQToken = " = ";
	private String CONSEQToken = "=";
	private boolean successInit = false;
	//Lavanya added
	public int rulesRipped = 0;

	public ForePostRuleClassifier(String arffFileName ) throws Exception
	{
		if( arffFileName == null ){
			throw new Exception("ARFF file name is null");
		}
		try {
			ArffLoader loader = new ArffLoader();
			loader.setFile(new File(arffFileName));
			data = loader.getDataSet();
			data.setClassIndex(data.numAttributes() - 1);
			consequent = data.attribute(data.numAttributes() - 1);
			successInit = true;
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Failed to validate ARFF file");
			successInit = false;
		}
	}
	
	public ForePostRuleClassifier()
	{
		
	}
	
	public boolean Classify() throws Exception
	{
//		logger.info("start to classify the rules by JRip!"); //QI
    	try {
    		if(!successInit)
    			return false;
    		this.setDebug(false);
    		this.setFolds(10);
    		this.setUsePruning(false);
			buildClassifier(data);
			ruleset = getRuleset();
			return( true );
		} catch (Exception e) {
			e.printStackTrace();
		}
		return( false );
		
	}

	public Rules ProcessRules(){
		Rule rule = null;
		Rules rules = new Rules();
		try {
			FPUtility.RuleLog("Learned " + String.valueOf(ruleset.size()) + " rules" );
//			logger.info("start parse the rules!"); //QI
			for (int i = 0; i < ruleset.size(); i++) {
				RipperRule rr = (RipperRule)ruleset.elementAt(i);
				FPUtility.RuleLog("Rule " + String.valueOf(i+1) + rr.toString(consequent) );
//				logger.info(rr.toString(consequent));//QI
//			System.out.println(rr.toString(consequent));
				rule = ParseRule( rr.toString(consequent) );// WHY QI
//				rule.setRuleText(rr.toString(consequent));
				if( rule != null )
					rules.AddRule(rule);
				rulesRipped++;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return rules;
	}
	
	private Rule ParseRule( String strRule ) throws Exception
	{
		Rule rule = null;
		
		if( strRule == null ){
			throw new Exception("rule string is null");
		}
		
		String [] entries = strRule.split(ArrowToken);
		if( entries == null ){
			FPUtility.getLogger().log(Level.FINE, "Rule cannot be tokenized: " + strRule );
			return( rule );
		}
		
		if( entries.length != 2 )
		{
			FPUtility.getLogger().log(Level.FINE, "Rule is not properly constructed: " + strRule );
			return( rule );
		}
		
		rule = new Rule();
		rule.setRuleText(entries[0]);
		ParseAntecedent( entries[0], rule );
		String Class = ParseConsequent( entries[1] );
		if( rule.GetTotalTerms() <= 0 || Class == null )
			rule = null;
		else
			rule.SetClass( Class );
		
		return( rule );
	}
	
	private String ParseConsequent( String con ) throws Exception
	{
		if( con == null ){
			throw new Exception("consequent string is null");
		}
		
		String [] entries = con.split(CONSEQToken);
		if( entries == null ){
			FPUtility.getLogger().log(Level.FINE, "Consequent cannot be tokenized: " + con );
		}
		
		if (entries.length != 2) 
			return( null );
		
		return( entries[1].trim() );
		
	}

	private void ParseAntecedent(String ant, Rule r) throws Exception
	{
		if( ant == null || r == null ){
			throw new Exception("antecedent string is null");
		}
		
		String [] entries = ant.split(AndToken);
		if( entries == null ){
			FPUtility.getLogger().log(Level.FINE, "Antecedent cannot be tokenized: " + ant );
		}
		
		if (entries.length<=0) 
			return;
		
		for( int i = 0; i < entries.length; i++ ){
			Term t = ParseTerm(entries[i]);
			if( t == null ){
				r.ResetTerms();
				return;
			}
			r.AddTerm( t );
		}
	}
	
	private Term ParseTerm( String tm ) throws Exception
	{
		Term.RELOPERATION op = Term.RELOPERATION.NOOP;
		Term term = null;
		String [] entries = null;
		if( tm == null ){
			throw new Exception("term string is null");
		}
		
		entries = tm.split(LEQToken);
		if( entries.length != 2 ){
			entries = tm.split(EQToken);
			if( entries.length != 2 ){
				entries = tm.split(GEQToken);
				if( entries.length != 2 ){
					FPUtility.getLogger().log(Level.FINE, "term string cannot be tokenized: " + tm );
					return( term );
				}
				else
					op = Term.RELOPERATION.GEQ;
			}
			else
				op = Term.RELOPERATION.EQUAL;
		}
		else
			op = Term.RELOPERATION.LEQ;
		
		if (entries.length<=0) 
			return( term );

		String attr = entries[0];
		int ind = entries[0].indexOf('(');
		if( ind >= 0 )
			attr = entries[0].substring(ind+1);
		attr = attr.trim();
		ind = entries[1].indexOf(')');
		String val = entries[1];
		if( ind > 0 )
			val = entries[1].substring(0, ind);
		val = val.trim();

		term = new Term( op, attr, val );
		
		return( term );
	}
}
