package forepost;
import java.util.*;
import java.util.logging.Level;
import java.io.*;

import org.apache.log4j.Logger;

import com.accenture.techlabs.assist.profile.ProfRepository;

public class ProfileParser implements ProfileStatistics 
{
	// QI
	private static final Logger logger = Logger.getLogger(ProfRepository.class);
	
	final static String token = "\\_\\|\\_", tokenEnd = "\\_\\|\\|\\_";
	final static String MTDENT = "MTDENT";
	final static String MTDRET = "MTDRET";
	final static String IMSRET = "IMSRET";
	final static String REQUEST = "REQENT";
	final static String JDBC = "JDBC";
	int countOfJDBC = 0;
	int Loop = 0;
	
	ExecutionStack execStack = new ExecutionStack();
	// edit by QI - begin
	//String ProfileClass = null;
	String ProfileClass = "Unassigned";
	// edit by QI - end
	LinkedList<Method> methods = new LinkedList<Method>();
	Hashtable <String, Integer> ThreadCount = new Hashtable <String, Integer>();
	ImsDataParser ims = new ImsDataParser();
	RequestParser req = new RequestParser();
	private RuleCountClass currProfileInfo = new RuleCountClass();

	public ProfileParser() throws Exception
	{
	}
	
	//
	//code addded by Aswathy Nair
	//
	public RuleCountClass getCurrProfileInfo() {
		return currProfileInfo;
	}

	//
	//code added by Aswathy Nair
	//
	public void setCurrProfileInfo(RuleCountClass currProfileInfo) {
		this.currProfileInfo.setRuleClass(currProfileInfo.getRuleClass());
		this.currProfileInfo.setRuleCount(currProfileInfo.getRuleCount());
	}
	
	public boolean Parse( String entry ) throws Exception
	{
		//logger.info("start parse profiles to learn rules!"); //	QI
		if( entry == null )
			throw new Exception( "The string is null" );

		Method method;
		String threadId = null, container = null, eventTime = null, methodparms = null;
		String transactionId, objectId, requestIMS, returnIMS, reqString, url;
		
		String [] entries = entry.split(token);
		if( entries == null )
			throw new Exception( "String is not tokenized" );
		
		String event = entries[0];
		if( IsEntry(event) || IsExit(event) ){
			threadId = entries[1];
			eventTime  = entries[2];
			container = entries[3];
			methodparms = entries[4];
			entries = entries[4].split(tokenEnd);
			if( entries == null )
				throw new Exception( "String is not tokenized" );
			methodparms = entries[0];
		} else if( IsData(event) ) {
//			System.out.println("Data entry is processed"); hide_focus
			threadId = entries[1];
			eventTime  = entries[2];
			objectId  = entries[3];
			transactionId  = entries[4];
			requestIMS  = entries[5];
			returnIMS  = entries[6];
			entries = returnIMS.split(tokenEnd);
			if( entries == null )
				throw new Exception( "String is not tokenized properly" );
			if( entries.length <= 0 )
				returnIMS = "";
			else
				returnIMS = entries[0];
			
			ims.Parse(returnIMS); 
		} else if( IsRequest(event) ) {
			//
			//Code modified by Aswathy Nair
			//
			
//			System.out.println("Request entry is processed"); hide_focus
			threadId = entries[1];
			eventTime  = entries[2];
			methodparms = entries[4];
			url = entries[6];
			reqString  = entries[7];
			entries = entries[7].split(tokenEnd);
			if( entries == null )
				throw new Exception( "String is not tokenized properly" );
			if( entries.length <= 0 )
				reqString = "";
			else
				reqString = entries[0];
			reqString = url + "?" + reqString; 
			req.Parse(reqString); 
		} /*//QI
			else if (IsJDBC (event)) {
			String fileJDBC = "C:\\Research\\codes\\JMeterScript\\Results\\JDBC_ " + countOfJDBC + ".txt";
			Writer output = null;
			File file = new File(fileJDBC);
			output = new BufferedWriter(new FileWriter(file));
			String outStr = "";
			
			Iterator iter = execStack.execStack.entrySet().iterator();
			while (iter.hasNext()) {
				Map.Entry mapEntry = (Map.Entry) iter.next();
				String key = (String) mapEntry.getKey();
				Stack<MethodCall> threadStack = (Stack<MethodCall>) mapEntry.getValue();
				//Stack<MethodCall> midStack = new Stack<MethodCall>();
				for (int i = 0; i < threadStack.size(); i++) {
					MethodCall met = threadStack.elementAt(i);
					outStr = "---------------------" + "\n" + "Thread ID: " + key + "\n" + "Method name and container: " + 
					met.method.getName() + "\n" + "Start time: " + met.start + "\n" 
					+ "End time: " + met.end + "\n";
					output.write(outStr);
				}
			}
			output.flush();
		    output.close();
		    countOfJDBC++;
		}*/

		if( IsEntry(event) ) {
			if( (method = GetMethod(container, methodparms)) == null ) {
				method = new Method( container, methodparms );
				methods.add(method);
			}
			execStack.AddMethodCall(threadId, Long.parseLong(eventTime), method);
			Integer tc = ThreadCount.get(threadId);
			if( tc == null ) {
				tc = new Integer(1);
			}
			else {
				tc = new Integer(tc.intValue());
			}
			ThreadCount.put(threadId, tc);
		} else if( IsExit(event) ) {
			method = GetMethod(container, methodparms);
			execStack.EndMethodCall(threadId, Long.parseLong(eventTime), method);
		} else if( IsData(event) ) {
			execStack.SetDataInfo(threadId, ims.getAttrs(), ims.getDatalen());
		}

//		ListAllMethods();
		
		return( true );
	}
	
	private boolean IsEntry(String event) {
		if( event != null && event.compareTo(MTDENT) == 0 ) {
			return( true );
		}
		return( false );
	}

	private boolean IsExit(String event) {
		if( event != null && event.compareTo(MTDRET) == 0 ) {
			return( true );
		}
		return( false );
	}

	private boolean IsData(String event) {
		if( event != null && event.compareTo(IMSRET) == 0 ) {
			return( true );
		}
		return( false );
	}
	
	private boolean IsRequest(String event) {
		if( event != null && event.compareTo(REQUEST) == 0 ) {
			return( true );
		}
		return( false );
	}
	
	private boolean IsJDBC (String event) {
		if( event != null && event.compareTo(JDBC) == 0) {
			return(true);
		}
		return(false);
	}
	
	
	public void SetProfileClass( String pclass ) throws Exception
	{
		if( pclass == null ) throw new Exception( "Profile class is null" );
		ProfileClass = pclass;
	}

	public String GetProfileClass(){
		return( ProfileClass );
	}
	
	public ArrayList<Hashtable<String, String>> getImsRuns() {
		return ims.getRuns();
	}

	public ArrayList<Hashtable<String, String>> getInputParamsRuns() {
		return req.getRuns();
	}
	
	public int GetMethods(){
		return( methods.size() );
	}

	public void MaxTime() throws Exception{
		ListIterator<Method> iter = methods.listIterator();
		while( iter.hasNext() )
		{
			Method lmethod = iter.next();
			if( lmethod != null ){
				lmethod.MaxTime();
			}
			else{
				throw new Exception( "Method object is null" );			
			}
		}
	}
	
	public int GetInvocations() throws Exception{
		int Invocations = 0;
		ListIterator<Method> iter = methods.listIterator();
		while( iter.hasNext() )
		{
			Method lmethod = iter.next();
			if( lmethod != null ){
				Invocations += lmethod.GetInvocations();
			}
			else{
				throw new Exception( "Method object is null" );			
			}
		}
		return( Invocations );
	}

	public int GetThreads() {
		return( ThreadCount.keySet().size() );
	}
	
	public int GetAttributes() throws Exception{
		int Attrs = 0;
		ListIterator<Method> iter = methods.listIterator();
		while( iter.hasNext() )
		{
			Method method = iter.next();
			if( method != null ){
				Attrs += method.GetAttributes();
			}
			else{
				throw new Exception( "Method object is null" );			
			}
		}
		return( Attrs );		
	}
	
	public int GetDatalen() throws Exception{
		int Datalen = 0;

		ListIterator<Method> iter = methods.listIterator();
		while( iter.hasNext() )
		{
			Method method = iter.next();
			if( method != null ){
				Datalen += method.GetDatalen();
			}
			else{
				throw new Exception( "Method object is null" );			
			}
		}
		return( Datalen );		
	}
	
	public double GetElapsedTime() throws Exception{
		double etime = 0.0, tm;
		
		ListIterator<Method> iter = methods.listIterator();
		while( iter.hasNext() )
		{
			Method method = iter.next();
			if( method != null ){
				tm = method.ComputeElapsedTime();
				if( tm > 0){
					etime += tm; 
				}
			}
			else{
				throw new Exception( "Method object is null" );			
			}
		}
		return( etime );
	}
	

	public double GetElapsedTimeRatio() throws Exception{
		if( ComputedValues.MAXTIME == 0 )
			return(0.0);
		try {
			return( GetElapsedTime()/ComputedValues.MAXTIME );
		} catch (Exception e) {
			throw new Exception( e );
		}
	}
	
	public void PrintAll(Writer output)throws Exception 
	{
		if( output == null ) throw new Exception( "Writer object is null" );
		try 
		{
			String outStr = null;
	
			output.write("A total of " + String.valueOf(methods.size()) + " method calls are found"+"\n");
			ListIterator<Method> iter = methods.listIterator();
			while( iter.hasNext() )
			{
				Method method = iter.next();
				if( method != null ){
					if( method.ComputeElapsedTime() > 0){
						outStr = "Method: " + method.getName() + ", Container: " + method.getContainer() + ", Elapsed: " + String.valueOf(method.GetElapsedTimeRatio()) + ", attributes: " + String.valueOf(method.GetAttributeRatio()) + ", data: " + String.valueOf(method.GetDatalengthRatio()) + "\n";
						output.write(outStr);
					}
				}
				else{
					throw new Exception( "Method object is null" );			
				}
			}
		} catch( Exception e ){
			e.printStackTrace();
			throw new Exception( "exception when writing signals" );
		}
	}
	
	public void ListAllMethods() throws Exception 
	{
		System.out.println("A total of " + String.valueOf(methods.size()) + " method calls are found" );
		FPUtility.getLogger().info("A total of " + String.valueOf(methods.size()) + " method calls are found");
		ListIterator<Method> iter = methods.listIterator();
		while( iter.hasNext() )
		{
			Method method = iter.next();
			if( method != null ){
				if( method.ComputeElapsedTime() > 0)
				{
					System.out.println("Method: " + method.getName() + ", Container: " + method.getContainer() + ", Elapsed: " + String.valueOf(method.ComputeElapsedTime()) );
					FPUtility.getLogger().info("Method: " + method.getName() + ", Container: " + method.getContainer() + ", Elapsed: " + String.valueOf(method.ComputeElapsedTime()));
				}
			}
			else{
				throw new Exception( "Method object is null" );			
			}
		}
	}

	public Method GetMethod(String container, String methodparms)
	{
		Method meth = null;
		ListIterator<Method> iter = Collections.unmodifiableList(methods).listIterator();
		while( iter.hasNext() )
		{
			meth = iter.next();
			if( meth != null ){
				if( meth != null && meth.Compare(container, methodparms)) {
					return( meth );
				}
			}
			else{
				try {
					FPUtility.getLogger().log(Level.SEVERE, "Method object is null" );
				} catch (Exception e) {
					e.printStackTrace();
				}			
			}
		}
		return( null );
	}
		
	public LinkedList<Method> getMethodVector() {
		return methods;
	}
	
	public int getLoop() {
		return Loop;
	}

	public void setLoop(int loop) {
		Loop = loop;
	}

	public String getProfileClass() {
		return ProfileClass;
	}

	public void setProfileClass(String profileClass) {
		ProfileClass = profileClass;
	}

	public void SetInputParamARFFMgr( NewArffManager arffMgr )
	{
		req.setDam(arffMgr);
	}

	public void SetDataARFFMgr( NewArffManager arffMgr )
	{
		ims.setDam(arffMgr);
	}
}

