package forepost;
import java.util.*;
import java.util.logging.Level;

public class ProfileTable 
{
	private LinkedList<MethodExecTimes> Methods = new LinkedList<MethodExecTimes>();
	private LinkedList <ProfileParser> profiles = new LinkedList <ProfileParser>();
	private String Class = null;
	
	public LinkedList<ProfileParser> getProfiles() {
		return profiles;
	}

	public ProfileTable( String Class ) throws Exception {
		if( Class == null )
			throw new Exception("Class of the profile table is null");
		this.Class = Class;
	}
	
	public void AddProfile( ProfileParser profile ) throws Exception
	{
		if( profile == null )
		{
			FPUtility.getLogger().log(Level.SEVERE, "Profile object is null when added to the profile table for the class ", Class );
			throw new Exception("Profile object is null");
		}
		
		if( profile.getProfileClass().equalsIgnoreCase(Class) )
		{
			FPUtility.getLogger().log(Level.FINE, "Profile is added to the table of the class " + Class );
			profiles.add(profile);
		}
		else
		{
			FPUtility.getLogger().log(Level.FINEST, "Profile is not added to the table, class mismatch for " + Class );
		}
	}
	
	public LinkedList<MethodExecTimes> GetListOfMethods() {
		return( Methods );
	}
	
	public void ProcessProfiles() throws Exception {
		if( profiles == null )
			throw new Exception("The profile vector object is null");
		
		if( profiles.size() <= 0 )
			throw new Exception("No profiles to process");
		
		GenerateUniqueMethodExecEntries();
		ComputeMethodExecCoefficient();
	}
		
	private void GenerateUniqueMethodExecEntries() throws Exception {
		LinkedList<Method> methods = null;
		ProfileParser pp = null;
		ListIterator<ProfileParser> iter = profiles.listIterator();
		while( iter.hasNext() )
		{
			pp = iter.next();
			if( pp == null ){
				FPUtility.getLogger().log(Level.FINE, "Unexpected null element in the profile vector" );
				throw new Exception("Unexpected null element in the profile vector");
			}
			
			methods = pp.getMethodVector();
			Method m = null;
			ListIterator<Method> iterMethod = methods.listIterator();
			while( iterMethod.hasNext() )
			{
				m = iterMethod.next();
				int index = FindMethod( m );
				if( index < 0 ){
					if( AddMethod(m, profiles.size()) < 0 )
						throw new Exception("MethodExecTimes object is not added properly");
				}
				else {
					MethodExecTimes method = Methods.get(index);
					method.AddMethod( m );
				}
			}
		}
	}

	private void ComputeMethodExecCoefficient() throws Exception {	
		ProfileParser pp = null;
		ListIterator<ProfileParser> iter = profiles.listIterator();
		while( iter.hasNext() )
		{
			pp = iter.next();
			if( pp == null ){
				FPUtility.getLogger().log(Level.FINE, "Unexpected null element in the profile vector" );
				throw new Exception("Unexpected null element in the profile vector");
			}

			LinkedList<Method> methods = pp.getMethodVector();
			Method m = null;
			ListIterator<Method> iterMethod = methods.listIterator();
			while( iterMethod.hasNext() )
			{
				m = iterMethod.next();
				int index = FindMethod( m );
				if( index < 0 ){
					FPUtility.getLogger().log(Level.SEVERE, "MethodExecTimes object is not added properly" );
					throw new Exception("MethodExecTimes object is not added properly");
				} else {
					MethodExecTimes method = Methods.get(index);
					if( method == null ){
						throw new Exception("MethodExecTimes object is null");
					}
					method.SetTime(iter.nextIndex()-1, m.GetElapsedTimeRatio()+m.GetAttributeRatio()+m.GetDatalengthRatio());
					if( m.GetAttributeRatio() > 0 || m.GetDatalengthRatio() > 0 )
					System.out.println("For method " + method.getName() + " values are " + String.valueOf(m.GetElapsedTimeRatio()) + "::" + 
							String.valueOf(m.GetAttributeRatio()) + "::" + String.valueOf(m.GetDatalengthRatio()) );
				}
			}
		}
	
	}
	
	private int AddMethod(Method m, int profSize) throws Exception {
		if( m == null || m.getContainer() == null || m.getName() == null )
			throw new Exception("Cannot add the method with null identifiers");
		MethodExecTimes mt = new MethodExecTimes(profSize, m); 
		Methods.add(mt);
		return( Methods.indexOf(mt) );
	}

	private int FindMethod( Method m ){
		int index = -1;
		MethodExecTimes method = null;
		for( int j = 0; j < Methods.size(); j++ ){
			method = Methods.get(j);
			if( m != null && method != null && method.Compare(m) ) {
				index = j;
				break;
			}
		}
		return( index );
	}
	
	public double [][] getMatrix() throws Exception 
	{
		int rows = profiles.size(), columns = Methods.size();
		
		double [][] matrix = new double [rows][columns];
		
		for( int row = 0; row < rows; row++ ){
			for( int col = 0; col < columns; col++ ){
				MethodExecTimes method = Methods.get(col);
				if( method == null ){
					throw new Exception("MethodExecTimes object is null");
				}
				ArrayList<Double> artime = method.GetTimes(); 
				matrix[row][col] = artime.get(row).doubleValue();
			}
		}
		
		return( matrix );
	}
}
