package forepost.utilities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class Rules implements Iterable<Rule>, Iterator<Rule>, Serializable 
{
	private ArrayList<Rule> Rules = new ArrayList<Rule>();
	private int current = 0;
	
	public Rules() {
		super();
	}
	
	public Rules (Rule rule) {
		AddRule(rule);
	}

	//get sorted iterator
	public Iterator<Rule> iterator() {
		current = 0;
		Collections.sort(Rules, new RuleComparator());
		return this;
	}

	public boolean hasNext() {
		return current < Rules.size();
	}

	public Rule next() {
		if( current >= Rules.size() )
			throw new NoSuchElementException();
		return Rules.get(current++);
	}

	public void remove() {
		if( current > Rules.size() || current <= 0 )
			throw new NoSuchElementException();
		
		Rules.remove(current-1);
	}

	public void AddRule(Rule rule) {
		Rules.add(rule);
	}

	public class RuleComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			int r1Terms = ((Rule) o1).GetTotalTerms();
			int r2Terms = ((Rule) o2).GetTotalTerms();
			
			if (r1Terms < r2Terms) 
				return 1;
			else if (r1Terms > r2Terms)
				return -1;
			else 
				return 0;
		}
	}
	
	public static Rules mockRules() {
		Rule r1 = new Rule(new Term(Term.RELOPERATION.EQUAL, "Location.insideCityLimit", "N"));
		r1.AddTerm(new Term(Term.RELOPERATION.EQUAL, "Location.visibleFromRoadOrNeighbor", "N"));
		Rule r2 = new Rule(new Term(Term.RELOPERATION.GREATER, "PreEligibilitySO.numberDogsInHouseHold", "2"));
		Rules rules = new Rules();
		rules.AddRule(r1);
		Rule r3 = new Rule(new Term(Term.RELOPERATION.EQUAL, "Coverages.deductiblePerOccurrence", "00500"));
		r3.AddTerm(new Term(Term.RELOPERATION.EQUAL, "Coverages.lossSettlementPersonalProp", "LSP  B2"));
		r3.AddTerm(new Term(Term.RELOPERATION.EQUAL, "Coverages.limitEachPerson", "1000"));
//		r2.AddTerm(new Term(Term.RELOPERATION.LEQ, "PreEligibilitySO.numberDogsInHouseHold", "4"));
		rules.AddRule(r2);
		rules.AddRule(r3);
		return rules;
	}
	
	public static void main(String[] args) {
		Rule r1 = new Rule(new Term(Term.RELOPERATION.GREATER, "CustomersSO.numberResidents", "2"));
		Rule r2 = new Rule(new Term(Term.RELOPERATION.GREATER, "CustomersSO.numberResidents", "2"));
		Rules rules = new Rules();
		rules.AddRule(r1);
		r2.AddTerm(new Term(Term.RELOPERATION.LEQ, "CustomersSO.numberResidents", "3"));
		rules.AddRule(r2);
		Iterator ri = rules.iterator();
		while (rules.hasNext()) {
			System.out.println(rules.next().GetTotalTerms());
		}
		
		System.out.println("------------");
/*		while (rules.iterator().hasNext()) {
			System.out.println(rules.next());
		}*/
	}
}
