package forepost.utilities;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class ScriptUtilities {
	
	public static TermTable convertRulesToTermTable(Rules rs) throws Exception{
		TermTable tables = new TermTable();
		while (rs.hasNext()) {
			tables.addRule(rs.next());
		}
		return tables;
	}
	
	public static List<String> calculatePossibleValues(String key, HashMap<String, ArrayList<Term>> activeRule, Map<String, List<String>> widgetValuesMap) {
		List<String> possibleValues = new ArrayList<String> ();
		List<Term> terms = activeRule.get(key.toLowerCase());
		List<String> allValues = widgetValuesMap.get(key.toLowerCase()); 
		//get each term for the widget
		for (int i = 0; i < terms.size(); i++) {
			Term.VALUETYPE vType = terms.get(i).getValueType();
			Term.RELOPERATION relOp = terms.get(i).getRop();
			String value = terms.get(i).getValue();
			List<String> tempValues = calculatePossibleValues(vType, relOp, value, allValues);
			if (i == 0) {
				possibleValues = tempValues;
			} else {
				possibleValues = intersect(possibleValues, tempValues);
			}
		}

		return possibleValues;
	} 
	
	private static List<String> calculatePossibleValues(Term.VALUETYPE vType, Term.RELOPERATION relOp, String value, List<String> values) {
		List<String> possibleValues = new ArrayList<String> ();
		if (vType == Term.VALUETYPE.INTVAL) {
			switch (relOp) {
				case EQUAL:
					possibleValues.add(value);
					break;
				case GEQ:
					for (int i = 0; i < values.size(); i ++) {
						if (Integer.parseInt(values.get(i)) >= Integer.parseInt(value)) {
							possibleValues.add(values.get(i));
						}
					}
					break;
				case GREATER:
					for (int i = 0; i < values.size(); i ++) {
						if (Integer.parseInt(values.get(i)) > Integer.parseInt(value)) {
							possibleValues.add(values.get(i));
						}
					}
					break;
				case LEQ:
					for (int i = 0; i < values.size(); i ++) {
						if (Integer.parseInt(values.get(i)) <= Integer.parseInt(value)) {
							possibleValues.add(values.get(i));
						}
					}
					break;
				case LESS:
					for (int i = 0; i < values.size(); i ++) {
						if (Integer.parseInt(values.get(i)) < Integer.parseInt(value)) {
							possibleValues.add(values.get(i));
						}
					}
					break;
				case NOTEQUAL:
					for (int i = 0; i < values.size(); i ++) {
						if (Integer.parseInt(values.get(i)) != Integer.parseInt(value)) {
							possibleValues.add(values.get(i));
						}
					}
					break;
			}

		} else {
			switch (relOp) {
			case EQUAL:
				possibleValues.add(value);
				break;
			case NOTEQUAL:
				for (int i = 0; i < values.size(); i ++) {
					if (!values.get(i).equalsIgnoreCase(value)) {
						possibleValues.add(values.get(i));
					}
				}
				break;
			}
		}
		return possibleValues;
	}
	
	private static List<String> intersect(List<String> list1, List<String> list2) {
		List<String> overlap = new ArrayList<String>();
		
		for (int i = 0; i < list1.size(); i ++) {
			for (int j = 0; j < list2.size(); j++) {
				if (list1.get(i).equalsIgnoreCase(list2.get(j))) {
					overlap.add(list1.get(i));
					break;
				}
			}
		}
		return overlap;
	}
    
	public static String insertCommas(String str) {
		boolean isInteger = false;
		try {
			Integer.parseInt(str);
			isInteger = true;
		} catch( NumberFormatException e ){
				return str;
		}
		if (isInteger && str.length() >=4 ) {
			return insertCommas(str.substring(0, str.length() - 3)) + "," + str.substring(str.length() - 3, str.length());
		} else {
			return str;
		}
	}
	
	
	public static void writeLog(HashMap<String, ArrayList<Term>> activeRule, int profile_ind, String fileName ) {
		String output = System.currentTimeMillis() + "\t" + Integer.toString(profile_ind);
		if (activeRule == null) {
			output +=  "\t" + "{NO_RULE}";
		} else {
			Set<String> widgets = activeRule.keySet();
			Iterator<String> itWidgets = widgets.iterator();
			while (itWidgets.hasNext()) {
				String widgetName = itWidgets.next();
				ArrayList<Term> terms = activeRule.get(widgetName);
				Iterator<Term> itTerm = terms.iterator();
				while (itTerm.hasNext()) {
					Term term = itTerm.next();
					output += "\t" + term.toString();
				}
			}
		}
		output += "\r\n";
		try {
			BufferedWriter Out = new BufferedWriter(new FileWriter(fileName, true));
			Out.append(output);
			Out.flush();
			Out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	public static void writeMap(HashMap<String, HashMap<String, String>> map, String fileName) {
		try {

				String output = "<ListMap>\n";
				Set<String> listboxes = map.keySet();
				Iterator<String> itListKeys = listboxes.iterator();
				while(itListKeys.hasNext()){
					String screen = itListKeys.next();
					HashMap<String, String> inputValuesMap = map.get(screen);
					output += "<ListBox name=\"";
					output += screen;
					output += "\">\n";
					Set<String> inputs = inputValuesMap.keySet();
					Iterator<String> itInputKeys = inputs.iterator();
					while(itInputKeys.hasNext()){
						String input = itInputKeys.next();
						output += "<key name=\"";
						output += input;
						output += "\">";
						String value = inputValuesMap.get(input);
						output += value;
						output += "</key>\n";
					}
					output += "</ListBox>\n";
				}
				output += "</ListMap>\n";
				BufferedWriter xmlOut = new BufferedWriter(new FileWriter(fileName));
				xmlOut.write(output);
				xmlOut.flush();
				xmlOut.close();
				
			} catch (Exception e) {
				e.printStackTrace();
			}
	
	}
	
	public static void main(String[] args) {
		Rules rules = Rules.mockRules();
		HashMap<String, ArrayList<Term>> activeRule = null;

		try {
			TermTable termtable = convertRulesToTermTable(rules);
			termtable.iterator();
			int counter = 0;
			while (termtable.hasNext()) {
				activeRule = termtable.next();
				writeLog(activeRule, ++counter, "PROFILE_RULE.log");
			}
		} catch (Exception e) {
			
		}

	}
	
}
