package forepost.utilities;

import java.util.*;

public class TermTable implements Iterable<HashMap<String, ArrayList<Term>>>, Iterator<HashMap<String, ArrayList<Term>>>
{
	private HashMap <String, ArrayList<Term>> TermTable = null;
	private ArrayList<HashMap<String, ArrayList<Term>>> RuleTable = new ArrayList<HashMap<String, ArrayList<Term>>>();
	private int currentRule = 0;

	public Map<String, ArrayList<Term>> getTermTable() {
		return TermTable;
	}

	private void addTerm(String widgetName, Term t) {
		// set all key to lower case
		if (t != null && widgetName != null) {
			if (TermTable.containsKey(widgetName.toLowerCase())) {
				ArrayList<Term> terms = TermTable.get(widgetName.toLowerCase());
				terms.add(t);
			}
			else {
				ArrayList<Term> terms = new ArrayList<Term>();
				terms.add(t);
				TermTable.put(widgetName.toLowerCase(), terms);
			}
		}
	}

	public void addRule(Rule rule) throws Exception 
	{
		if( rule == null ){
			throw new Exception( "rule object is null");
		}
		
		TermTable = new HashMap<String, ArrayList<Term>> ();
		Iterator<Term> termIter = rule.iterator();
		while( termIter.hasNext() ){
			Term t = termIter.next();
			addTerm(t.getInputName(), t);
		}
		RuleTable.add(TermTable);
	}
	
	public Iterator<HashMap<String, ArrayList<Term>>> iterator() {
		currentRule = 0;
		Collections.sort(RuleTable, new RuleComparator());
		return this;
	}

	public boolean hasNext() {
		return currentRule < RuleTable.size();
	}

	public HashMap<String, ArrayList<Term>> next() {
		if( currentRule >= RuleTable.size() )
			throw new NoSuchElementException();
		return RuleTable.get(currentRule++);
	}

	public void remove() {
		if( currentRule > RuleTable.size() || currentRule <= 0 )
			throw new NoSuchElementException();
		
		RuleTable.remove(currentRule-1);
	}
	
	public class RuleComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			int r1Terms = ((HashMap<String, ArrayList<Term>>) o1).size();
			int r2Terms = ((HashMap<String, ArrayList<Term>>) o2).size();
			
			if (r1Terms < r2Terms) 
				return 1;
			else if (r1Terms > r2Terms)
				return -1;
			else 
				return 0;
		}
	}
}
